@@ -78,6 +78,7 @@ from daft.io import (
     DataCatalogType,
     from_glob_path,
     read_csv,
+    read_deltalake,
     read_delta_lake,
     read_hudi,
     read_iceberg,
@@ -103,6 +104,7 @@ __all__ = [
     "read_parquet",
     "read_hudi",
     "read_iceberg",
+    "read_deltalake",
     "read_delta_lake",
     "read_sql",
     "read_lance",
