@@ -193,6 +193,13 @@ class DataType:
         """Create a Date DataType: A date with a year, month and day"""
         return cls._from_pydatatype(PyDataType.date())
 
+    @classmethod
+    def time(cls, timeunit: TimeUnit | str) -> DataType:
+        """Time DataType. Supported timeunits are "us", "ns"."""
+        if isinstance(timeunit, str):
+            timeunit = TimeUnit.from_str(timeunit)
+        return cls._from_pydatatype(PyDataType.time(timeunit._timeunit))
+
     @classmethod
     def timestamp(cls, timeunit: TimeUnit | str, timezone: str | None = None) -> DataType:
         """Timestamp DataType."""
@@ -229,10 +236,22 @@ class DataType:
             raise ValueError("The size for a fixed-size list must be a positive integer, but got: ", size)
         return cls._from_pydatatype(PyDataType.fixed_size_list(dtype._dtype, size))
 
+    @classmethod
+    def map(cls, key_type: DataType, value_type: DataType) -> DataType:
+        """Create a Map DataType: A map is a nested type of key-value pairs that is implemented as a list of structs with two fields, key and value.
+        Args:
+            key_type: DataType of the keys in the map
+            value_type: DataType of the values in the map
+        """
+        return cls._from_pydatatype(PyDataType.map(key_type._dtype, value_type._dtype))
+
     @classmethod
     def struct(cls, fields: dict[str, DataType]) -> DataType:
         """Create a Struct DataType: a nested type which has names mapped to child types
 
+        Example:
+        >>> DataType.struct({"name": DataType.string(), "age": DataType.int64()})
+
         Args:
             fields: Nested fields of the Struct
         """
@@ -269,7 +288,7 @@ class DataType:
         of the image array.
 
         If the height, width, or mode may vary across images in the array, leaving these fields unspecified when
-        creating this type will cause Daft to respresent this image array as a heterogeneous collection of images,
+        creating this type will cause Daft to represent this image array as a heterogeneous collection of images,
         where each image can have a different mode, height, and width. This is much more flexible, but will result
         in a less compact representation and may be make some operations less efficient.
 
@@ -359,6 +378,11 @@ class DataType:
             return cls.decimal128(arrow_type.precision, arrow_type.scale)
         elif pa.types.is_date32(arrow_type):
             return cls.date()
+        elif pa.types.is_date64(arrow_type):
+            return cls.timestamp(TimeUnit.ms())
+        elif pa.types.is_time64(arrow_type):
+            timeunit = TimeUnit.from_str(pa.type_for_alias(str(arrow_type)).unit)
+            return cls.time(timeunit)
         elif pa.types.is_timestamp(arrow_type):
             timeunit = TimeUnit.from_str(arrow_type.unit)
             return cls.timestamp(timeunit=timeunit, timezone=arrow_type.tz)
@@ -377,6 +401,12 @@ class DataType:
             assert isinstance(arrow_type, pa.StructType)
             fields = [arrow_type[i] for i in range(arrow_type.num_fields)]
             return cls.struct({field.name: cls.from_arrow_type(field.type) for field in fields})
+        elif pa.types.is_map(arrow_type):
+            assert isinstance(arrow_type, pa.MapType)
+            return cls.map(
+                key_type=cls.from_arrow_type(arrow_type.key_type),
+                value_type=cls.from_arrow_type(arrow_type.item_type),
+            )
         elif _RAY_DATA_EXTENSIONS_AVAILABLE and isinstance(arrow_type, tuple(_TENSOR_EXTENSION_TYPES)):
             scalar_dtype = cls.from_arrow_type(arrow_type.scalar_type)
             shape = arrow_type.shape if isinstance(arrow_type, ArrowTensorType) else None
@@ -454,12 +484,24 @@ class DataType:
     def _is_fixed_shape_image_type(self) -> builtins.bool:
         return self._dtype.is_fixed_shape_image()
 
+    def _is_numeric_type(self) -> builtins.bool:
+        return self._dtype.is_numeric()
+
+    def _is_map(self) -> builtins.bool:
+        return self._dtype.is_map()
+
     def _is_logical_type(self) -> builtins.bool:
         return self._dtype.is_logical()
 
     def _is_temporal_type(self) -> builtins.bool:
         return self._dtype.is_temporal()
 
+    def _should_cast_to_python(self) -> builtins.bool:
+        # NOTE: This is used to determine if we should cast a column to a Python object type when converting to PyList.
+        # Map is a logical type, but we don't want to cast it to Python because the underlying physical type is a List,
+        # which we can handle without casting to Python.
+        return self._is_logical_type() and not self._is_map()
+
     def __repr__(self) -> str:
         return self._dtype.__repr__()
 
