@@ -1,10 +1,20 @@
 from __future__ import annotations
 
 import builtins
+import math
 import os
 from datetime import date, datetime, time
 from decimal import Decimal
-from typing import TYPE_CHECKING, Any, Callable, Iterable, Iterator, Literal, TypeVar, overload
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Iterable,
+    Iterator,
+    Literal,
+    TypeVar,
+    overload,
+)
 
 import pyarrow as pa
 
@@ -26,11 +36,9 @@ from daft.series import Series, item_to_series
 
 if TYPE_CHECKING:
     from daft.io import IOConfig
-
 # This allows Sphinx to correctly work against our "namespaced" accessor functions by overriding @property to
 # return a class instance of the namespace instead of a property object.
-property = property
-if os.getenv("DAFT_SPHINX_BUILD") == "1":
+elif os.getenv("DAFT_SPHINX_BUILD") == "1":
     from typing import Any
 
     # when building docs (with Sphinx) we need access to the functions
@@ -95,7 +103,24 @@ def col(name: str) -> Expression:
     """Creates an Expression referring to the column with the provided name
 
     Example:
-        >>> col("x")
+        >>> import daft
+        >>> df = daft.from_pydict({"x": [1, 2, 3], "y": [4, 5, 6]})
+        >>> df = df.select(daft.col("x"))
+        >>> df.show()
+        ╭───────╮
+        │ x     │
+        │ ---   │
+        │ Int64 │
+        ╞═══════╡
+        │ 1     │
+        ├╌╌╌╌╌╌╌┤
+        │ 2     │
+        ├╌╌╌╌╌╌╌┤
+        │ 3     │
+        ╰───────╯
+        <BLANKLINE>
+        (Showing first 3 of 3 rows)
+
 
     Args:
         name: Name of column
@@ -394,6 +419,16 @@ class Expression:
         expr = self._expr.arctan()
         return Expression._from_pyexpr(expr)
 
+    def arctan2(self, other: Expression) -> Expression:
+        """Calculates the four quadrant arctangent of coordinates (y, x), in radians (``expr_y.arctan2(expr_x)``)
+
+        * ``x = 0``, ``y = 0``: ``0``
+        * ``x >= 0``: ``[-pi/2, pi/2]``
+        * ``y >= 0``: ``(pi/2, pi]``
+        * ``y < 0``: ``(-pi, -pi/2)``"""
+        expr = Expression._to_expression(other)
+        return Expression._from_pyexpr(self._expr.arctan2(expr._expr))
+
     def radians(self) -> Expression:
         """The elementwise radians of a numeric expression (``expr.radians()``)"""
         expr = self._expr.radians()
@@ -414,6 +449,15 @@ class Expression:
         expr = self._expr.log10()
         return Expression._from_pyexpr(expr)
 
+    def log(self, base: float = math.e) -> Expression:  # type: ignore
+        """The elementwise log with given base, of a numeric expression (``expr.log(base = math.e)``)
+        Args:
+            base: The base of the logarithm. Defaults to e.
+        """
+        assert isinstance(base, (int, float)), f"base must be an int or float, but {type(base)} was provided."
+        expr = self._expr.log(float(base))
+        return Expression._from_pyexpr(expr)
+
     def ln(self) -> Expression:
         """The elementwise natural log of a numeric expression (``expr.ln()``)"""
         expr = self._expr.ln()
@@ -660,6 +704,22 @@ class Expression:
         expr = self._expr.is_in(other._expr)
         return Expression._from_pyexpr(expr)
 
+    def between(self, lower: Any, upper: Any) -> Expression:
+        """Checks if values in the Expression are between lower and upper, inclusive.
+
+        Example:
+            >>> # [1, 2, 3, 4] -> [True, True, False, False]
+            >>> col("x").between(1, 2)
+
+        Returns:
+            Expression: Boolean Expression indicating whether values are between lower and upper, inclusive.
+        """
+        lower = Expression._to_expression(lower)
+        upper = Expression._to_expression(upper)
+
+        expr = self._expr.between(lower._expr, upper._expr)
+        return Expression._from_pyexpr(expr)
+
     def name(self) -> builtins.str:
         return self._expr.name()
 
@@ -770,13 +830,70 @@ class ExpressionFloatNamespace(ExpressionNamespace):
 
         Example:
             >>> # [1., None, NaN] -> [False, None, True]
-            >>> col("x").is_nan()
+            >>> col("x").float.is_nan()
 
         Returns:
             Expression: Boolean Expression indicating whether values are invalid.
         """
         return Expression._from_pyexpr(self._expr.is_nan())
 
+    def is_inf(self) -> Expression:
+        """Checks if values in the Expression are Infinity.
+
+        .. NOTE::
+            Nulls will be propagated! I.e. this operation will return a null for null values.
+
+        Example:
+            >>> # [-float("inf"), 0., float("inf"), None] -> [True, False, True, None]
+            >>> col("x").float.is_inf()
+
+        Returns:
+            Expression: Boolean Expression indicating whether values are Infinity.
+        """
+        return Expression._from_pyexpr(self._expr.is_inf())
+
+    def not_nan(self) -> Expression:
+        """Checks if values are not NaN (a special float value indicating not-a-number)
+
+        .. NOTE::
+            Nulls will be propagated! I.e. this operation will return a null for null values.
+
+        Example:
+            >>> # [1., None, NaN] -> [True, None, False]
+            >>> col("x").not_nan()
+
+        Returns:
+            Expression: Boolean Expression indicating whether values are not invalid.
+        """
+        return Expression._from_pyexpr(self._expr.not_nan())
+
+    def fill_nan(self, fill_value: Expression) -> Expression:
+        """Fills NaN values in the Expression with the provided fill_value
+
+        Example:
+            >>> df = daft.from_pydict({"data": [1.1, float("nan"), 3.3]})
+            >>> df = df.with_column("filled", df["data"].float.fill_nan(2.2))
+            >>> df.show()
+            ╭─────────┬─────────╮
+            │ data    ┆ filled  │
+            │ ---     ┆ ---     │
+            │ Float64 ┆ Float64 │
+            ╞═════════╪═════════╡
+            │ 1.1     ┆ 1.1     │
+            ├╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌┤
+            │ NaN     ┆ 2.2     │
+            ├╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌┤
+            │ 3.3     ┆ 3.3     │
+            ╰─────────┴─────────╯
+
+        Returns:
+            Expression: Expression with Nan values filled with the provided fill_value
+        """
+
+        fill_value = Expression._to_expression(fill_value)
+        expr = self._expr.fill_nan(fill_value._expr)
+        return Expression._from_pyexpr(expr)
+
 
 class ExpressionDatetimeNamespace(ExpressionNamespace):
     def date(self) -> Expression:
@@ -1155,7 +1272,12 @@ class ExpressionStringNamespace(ExpressionNamespace):
         pattern_expr = Expression._to_expression(pattern)
         return Expression._from_pyexpr(self._expr.utf8_extract_all(pattern_expr._expr, index))
 
-    def replace(self, pattern: str | Expression, replacement: str | Expression, regex: bool = False) -> Expression:
+    def replace(
+        self,
+        pattern: str | Expression,
+        replacement: str | Expression,
+        regex: bool = False,
+    ) -> Expression:
         """Replaces all occurrences of a pattern in a string column with a replacement string. The pattern can be a literal string or a regex pattern.
 
         Example:
@@ -1394,6 +1516,97 @@ class ExpressionStringNamespace(ExpressionNamespace):
         pattern_expr = Expression._to_expression(pattern)
         return Expression._from_pyexpr(self._expr.utf8_ilike(pattern_expr._expr))
 
+    def substr(self, start: int | Expression, length: int | Expression | None = None) -> Expression:
+        """Extract a substring from a string, starting at a specified index and extending for a given length.
+
+        .. NOTE::
+            If `length` is not provided, the substring will include all characters from `start` to the end of the string.
+
+        Example:
+            >>> col("x").str.substr(2, 2)
+
+        Returns:
+            Expression: A String expression representing the extracted substring.
+        """
+        start_expr = Expression._to_expression(start)
+        length_expr = Expression._to_expression(length)
+        return Expression._from_pyexpr(self._expr.utf8_substr(start_expr._expr, length_expr._expr))
+
+    def to_date(self, format: str) -> Expression:
+        """Converts a string to a date using the specified format
+
+        .. NOTE::
+            The format must be a valid date format string.
+            See: https://docs.rs/chrono/latest/chrono/format/strftime/index.html
+
+        Example:
+            >>> df = daft.from_pydict({"x": ["2021-01-01", "2021-01-02", None]})
+            >>> df = df.with_column("date", df["x"].str.to_date("%Y-%m-%d"))
+            >>> df.show()
+            ╭────────────┬────────────╮
+            │ x          ┆ date       │
+            │ ---        ┆ ---        │
+            │ Utf8       ┆ Date       │
+            ╞════════════╪════════════╡
+            │ 2021-01-01 ┆ 2021-01-01 │
+            ├╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌┤
+            │ 2021-01-02 ┆ 2021-01-02 │
+            ├╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌┤
+            │ None       ┆ None       │
+            ╰────────────┴────────────╯
+
+
+        Returns:
+            Expression: a Date expression which is parsed by given format
+        """
+        return Expression._from_pyexpr(self._expr.utf8_to_date(format))
+
+    def to_datetime(self, format: str, timezone: str | None = None) -> Expression:
+        """Converts a string to a datetime using the specified format and timezone
+
+        .. NOTE::
+            The format must be a valid datetime format string.
+            See: https://docs.rs/chrono/latest/chrono/format/strftime/index.html
+
+        Example:
+            >>> df = daft.from_pydict({"x": ["2021-01-01 00:00:00.123", "2021-01-02 12:30:00.456", None]})
+            >>> df = df.with_column("datetime", df["x"].str.to_datetime("%Y-%m-%d %H:%M:%S%.3f"))
+            >>> df.show()
+            ╭─────────────────────────┬───────────────────────────────╮
+            │ x                       ┆ datetime                      │
+            │ ---                     ┆ ---                           │
+            │ Utf8                    ┆ Timestamp(Milliseconds, None) │
+            ╞═════════════════════════╪═══════════════════════════════╡
+            │ 2021-01-01 00:00:00.123 ┆ 2021-01-01 00:00:00.123       │
+            ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+            │ 2021-01-02 12:30:00.456 ┆ 2021-01-02 12:30:00.456       │
+            ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+            │ None                    ┆ None                          │
+            ╰─────────────────────────┴───────────────────────────────╯
+
+            If a timezone is provided, the datetime will be parsed in that timezone
+
+            >>> df = daft.from_pydict({"x": ["2021-01-01 00:00:00.123 +0800", "2021-01-02 12:30:00.456 +0800", None]})
+            >>> df = df.with_column("datetime", df["x"].str.to_datetime("%Y-%m-%d %H:%M:%S%.3f %z", timezone="Asia/Shanghai"))
+            >>> df.show()
+            ╭───────────────────────────────┬────────────────────────────────────────────────╮
+            │ x                             ┆ datetime                                       │
+            │ ---                           ┆ ---                                            │
+            │ Utf8                          ┆ Timestamp(Milliseconds, Some("Asia/Shanghai")) │
+            ╞═══════════════════════════════╪════════════════════════════════════════════════╡
+            │ 2021-01-01 00:00:00.123 +0800 ┆ 2021-01-01 00:00:00.123 CST                    │
+            ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+            │ 2021-01-02 12:30:00.456 +0800 ┆ 2021-01-02 12:30:00.456 CST                    │
+            ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+            │ None                          ┆ None                                           │
+            ╰───────────────────────────────┴────────────────────────────────────────────────╯
+
+
+        Returns:
+            Expression: a DateTime expression which is parsed by given format and timezone
+        """
+        return Expression._from_pyexpr(self._expr.utf8_to_datetime(format, timezone))
+
 
 class ExpressionListNamespace(ExpressionNamespace):
     def join(self, delimiter: str | Expression) -> Expression:
@@ -1568,7 +1781,10 @@ class ExpressionsProjection(Iterable[Expression]):
 
         return len(self._output_name_to_exprs) == len(other._output_name_to_exprs) and all(
             (s.name() == o.name()) and expr_structurally_equal(s, o)
-            for s, o in zip(self._output_name_to_exprs.values(), other._output_name_to_exprs.values())
+            for s, o in zip(
+                self._output_name_to_exprs.values(),
+                other._output_name_to_exprs.values(),
+            )
         )
 
     def union(self, other: ExpressionsProjection, rename_dup: str | None = None) -> ExpressionsProjection:
