@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import dataclasses
 import logging
+import os
 import pathlib
 import sys
 import urllib.parse
@@ -270,7 +271,20 @@ def _infer_filesystem(
     ###
     elif protocol in {"az", "abfs", "abfss"}:
         fsspec_fs_cls = get_filesystem_class(protocol)
-        fsspec_fs = fsspec_fs_cls()
+
+        if io_config is not None:
+            # TODO: look into support for other AzureConfig parameters
+            fsspec_fs = fsspec_fs_cls(
+                account_name=io_config.azure.storage_account,
+                account_key=io_config.azure.access_key,
+                sas_token=io_config.azure.sas_token,
+                tenant_id=io_config.azure.tenant_id,
+                client_id=io_config.azure.client_id,
+                client_secret=io_config.azure.client_secret,
+                anon=io_config.azure.anonymous,
+            )
+        else:
+            fsspec_fs = fsspec_fs_cls()
         resolved_filesystem, resolved_path = pafs_resolve_filesystem_and_path(path, fsspec_fs)
         resolved_path = resolved_filesystem.normalize_path(_unwrap_protocol(resolved_path))
         return resolved_path, resolved_filesystem
@@ -320,3 +334,19 @@ def glob_path_with_stats(
         num_rows.append(infos.get("rows"))
 
     return FileInfos.from_infos(file_paths=file_paths, file_sizes=file_sizes, num_rows=num_rows)
+
+
+###
+# Path joining
+###
+
+
+def join_path(fs: FileSystem, base_path: str, *sub_paths: str) -> str:
+    """
+    Join a base path with sub-paths using the appropriate path separator
+    for the given filesystem.
+    """
+    if isinstance(fs, LocalFileSystem):
+        return os.path.join(base_path, *sub_paths)
+    else:
+        return f"{base_path.rstrip('/')}/{'/'.join(sub_paths)}"
