@@ -7,10 +7,11 @@ from daft.daft import (
     GCSConfig,
     IOConfig,
     S3Config,
+    S3Credentials,
     set_io_pool_num_threads,
 )
 from daft.io._csv import read_csv
-from daft.io._delta_lake import read_delta_lake
+from daft.io._delta_lake import read_deltalake, read_delta_lake
 from daft.io._hudi import read_hudi
 from daft.io._iceberg import read_iceberg
 from daft.io._json import read_json
@@ -42,11 +43,13 @@ __all__ = [
     "read_parquet",
     "read_hudi",
     "read_iceberg",
+    "read_deltalake",
     "read_delta_lake",
     "read_lance",
     "read_sql",
     "IOConfig",
     "S3Config",
+    "S3Credentials",
     "AzureConfig",
     "GCSConfig",
     "set_io_pool_num_threads",
