@@ -68,7 +68,8 @@ class Series:
                 storage_series = Series.from_arrow(array.storage, name=name)
                 series = storage_series.cast(
                     DataType.fixed_size_list(
-                        DataType.from_arrow_type(array.type.scalar_type), int(np.prod(array.type.shape))
+                        DataType.from_arrow_type(array.type.scalar_type),
+                        int(np.prod(array.type.shape)),
                     )
                 )
                 return series.cast(DataType.from_arrow_type(array.type))
@@ -256,7 +257,10 @@ class Series:
                 storage = arrow_series.storage
                 list_size = storage.type.list_size
                 storage = pa.ListArray.from_arrays(
-                    pa.array(list(range(0, (len(arrow_series) + 1) * list_size, list_size)), pa.int32()),
+                    pa.array(
+                        list(range(0, (len(arrow_series) + 1) * list_size, list_size)),
+                        pa.int32(),
+                    ),
                     storage.values,
                 )
                 return pa.ExtensionArray.from_storage(pyarrow_dtype, storage)
@@ -392,6 +396,12 @@ class Series:
         """The elementwise arc tangent of a numeric series"""
         return Series._from_pyseries(self._series.arctan())
 
+    def arctan2(self, other: Series) -> Series:
+        """Calculates the four quadrant arctangent of coordinates (y, x)"""
+        if not isinstance(other, Series):
+            raise TypeError(f"expected another Series but got {type(other)}")
+        return Series._from_pyseries(self._series.arctan2(other._series))
+
     def radians(self) -> Series:
         """The elementwise radians of a numeric series"""
         return Series._from_pyseries(self._series.radians())
@@ -408,6 +418,13 @@ class Series:
         """The elementwise log10 of a numeric series"""
         return Series._from_pyseries(self._series.log10())
 
+    def log(self, base: float) -> Series:
+        """The elementwise log with given base, of a numeric series.
+        Args:
+            base: The base of the logarithm.
+        """
+        return Series._from_pyseries(self._series.log(base))
+
     def ln(self) -> Series:
         """The elementwise ln of a numeric series"""
         return Series._from_pyseries(self._series.ln())
@@ -628,6 +645,18 @@ class SeriesFloatNamespace(SeriesNamespace):
     def is_nan(self) -> Series:
         return Series._from_pyseries(self._series.is_nan())
 
+    def is_inf(self) -> Series:
+        return Series._from_pyseries(self._series.is_inf())
+
+    def not_nan(self) -> Series:
+        return Series._from_pyseries(self._series.not_nan())
+
+    def fill_nan(self, fill_value: Series) -> Series:
+        if not isinstance(fill_value, Series):
+            raise ValueError(f"expected another Series but got {type(fill_value)}")
+        assert self._series is not None and fill_value._series is not None
+        return Series._from_pyseries(self._series.fill_nan(fill_value._series))
+
 
 class SeriesStringNamespace(SeriesNamespace):
     def endswith(self, suffix: Series) -> Series:
@@ -766,6 +795,31 @@ class SeriesStringNamespace(SeriesNamespace):
         assert self._series is not None and pattern._series is not None
         return Series._from_pyseries(self._series.utf8_ilike(pattern._series))
 
+    def to_date(self, format: str) -> Series:
+        if not isinstance(format, str):
+            raise ValueError(f"expected str for format but got {type(format)}")
+        assert self._series is not None
+        return Series._from_pyseries(self._series.utf8_to_date(format))
+
+    def to_datetime(self, format: str, timezone: str | None = None) -> Series:
+        if not isinstance(format, str):
+            raise ValueError(f"expected str for format but got {type(format)}")
+        if timezone is not None and not isinstance(timezone, str):
+            raise ValueError(f"expected str for timezone but got {type(timezone)}")
+        assert self._series is not None
+        return Series._from_pyseries(self._series.utf8_to_datetime(format, timezone))
+
+    def substr(self, start: Series, length: Series | None = None) -> Series:
+        if not isinstance(start, Series):
+            raise ValueError(f"expected another Series but got {type(start)}")
+        if length is not None and not isinstance(length, Series):
+            raise ValueError(f"expected another Series but got {type(length)}")
+        if length is None:
+            length = Series.from_arrow(pa.array([None]))
+
+        assert self._series is not None and start._series is not None
+        return Series._from_pyseries(self._series.utf8_substr(start._series, length._series))
+
 
 class SeriesDateNamespace(SeriesNamespace):
     def date(self) -> Series:
