@@ -316,6 +316,7 @@ def read_csv(
                 delimiter=csv_options.delimiter,
                 double_quote=csv_options.double_quote,
                 quote=csv_options.quote,
+                allow_variable_columns=csv_options.allow_variable_columns,
                 escape_char=csv_options.escape_char,
                 comment=csv_options.comment,
             )
@@ -652,40 +653,49 @@ def write_deltalake(
     mp: MicroPartition,
     large_dtypes: bool,
     base_path: str,
-    current_version: int,
+    version: int,
     io_config: IOConfig | None = None,
 ):
+    import json
+    from datetime import datetime
+
+    import deltalake
     from deltalake.schema import convert_pyarrow_table
     from deltalake.writer import (
         AddAction,
         DeltaJSONEncoder,
         DeltaStorageHandler,
-        get_file_stats_from_metadata,
         get_partitions_from_path,
-        try_get_table_and_table_uri,
     )
+    from packaging.version import parse
     from pyarrow.fs import PyFileSystem
 
-    from daft.delta_lake.delta_lake_storage_function import (
-        _storage_config_to_storage_options,
-    )
+    from daft.io.object_store_options import io_config_to_storage_options
+    from daft.utils import ARROW_VERSION
+
+    protocol = get_protocol_from_path(base_path)
+    canonicalized_protocol = canonicalize_protocol(protocol)
 
     data_files: list[AddAction] = []
 
+    # added to get_file_stats_from_metadata in deltalake v0.17.4: non-optional "num_indexed_cols" argument
+    # https://github.com/delta-io/delta-rs/blob/353e08be0202c45334dcdceee65a8679f35de710/python/deltalake/writer.py#L725
+    if parse(deltalake.__version__) < parse("0.17.4"):
+        get_file_stats_from_metadata = deltalake.writer.get_file_stats_from_metadata
+    else:
+
+        def get_file_stats_from_metadata(metadata):
+            deltalake.writer.get_file_stats_from_metadata(metadata, -1)
+
     def file_visitor(written_file: Any) -> None:
         path, partition_values = get_partitions_from_path(written_file.path)
         stats = get_file_stats_from_metadata(written_file.metadata)
 
-        import json
-        from datetime import datetime
-
-        from daft.utils import ARROW_VERSION
-
         # PyArrow added support for written_file.size in 9.0.0
         if ARROW_VERSION >= (9, 0, 0):
             size = written_file.size
-        elif filesystem is not None:
-            size = filesystem.get_file_info([path])[0].size
+        elif fs is not None:
+            size = fs.get_file_info([path])[0].size
         else:
             size = 0
 
@@ -700,38 +710,47 @@ def write_deltalake(
             )
         )
 
+    is_local_fs = canonicalized_protocol == "file"
+
     io_config = get_context().daft_planning_config.default_io_config if io_config is None else io_config
-    storage_config = StorageConfig.native(NativeStorageConfig(False, io_config))
-    storage_options = _storage_config_to_storage_options(storage_config, base_path)
-    table, table_uri = try_get_table_and_table_uri(base_path, storage_options)
-    filesystem = PyFileSystem(DeltaStorageHandler(table_uri, storage_options))
+    storage_options = io_config_to_storage_options(io_config, base_path)
+    fs = PyFileSystem(DeltaStorageHandler(base_path, storage_options))
 
     arrow_table = mp.to_arrow()
     arrow_batch = convert_pyarrow_table(arrow_table, large_dtypes)
 
     execution_config = get_context().daft_execution_config
-    MAX_OPEN_FILE = execution_config.parquet_max_open_files
-    MAX_ROWS_PER_FILE = execution_config.parquet_max_rows_per_file
-    MIN_ROWS_PER_GROUP = execution_config.parquet_min_rows_per_group
-    MAX_ROWS_PER_GROUP = execution_config.parquet_max_rows_per_group
 
-    file_options = pads.ParquetFileFormat().make_write_options(use_compliant_nested_type=False)
+    target_row_group_size = execution_config.parquet_target_row_group_size
+    inflation_factor = execution_config.parquet_inflation_factor
+    target_file_size = execution_config.parquet_target_filesize
 
-    pads.write_dataset(
-        arrow_batch,
-        base_dir="/",
-        basename_template=f"{current_version + 1}-{uuid4()}-{{i}}.parquet",
-        format="parquet",
-        partitioning=None,
+    size_bytes = arrow_table.nbytes
+
+    target_num_files = max(math.ceil(size_bytes / target_file_size / inflation_factor), 1)
+    num_rows = len(arrow_table)
+
+    rows_per_file = max(math.ceil(num_rows / target_num_files), 1)
+
+    target_row_groups = max(math.ceil(size_bytes / target_row_group_size / inflation_factor), 1)
+    rows_per_row_group = max(min(math.ceil(num_rows / target_row_groups), rows_per_file), 1)
+
+    opts = pads.ParquetFileFormat().make_write_options(use_compliant_nested_type=False)
+
+    format = pads.ParquetFileFormat()
+
+    _write_tabular_arrow_table(
+        arrow_table=arrow_batch,
         schema=None,
+        full_path="/",
+        format=format,
+        opts=opts,
+        fs=fs,
+        rows_per_file=rows_per_file,
+        rows_per_row_group=rows_per_row_group,
+        create_dir=is_local_fs,
         file_visitor=file_visitor,
-        existing_data_behavior="overwrite_or_ignore",
-        file_options=file_options,
-        max_open_files=MAX_OPEN_FILE,
-        max_rows_per_file=MAX_ROWS_PER_FILE,
-        min_rows_per_group=MIN_ROWS_PER_GROUP,
-        max_rows_per_group=MAX_ROWS_PER_GROUP,
-        filesystem=filesystem,
+        version=version,
     )
 
     return MicroPartition.from_pydict({"data_file": Series.from_pylist(data_files, name="data_file", pyobj="force")})
@@ -748,6 +767,7 @@ def _write_tabular_arrow_table(
     rows_per_row_group: int,
     create_dir: bool,
     file_visitor: Callable | None,
+    version: int | None = None,
 ):
     kwargs = dict()
 
@@ -761,11 +781,16 @@ def _write_tabular_arrow_table(
     if ARROW_VERSION >= (8, 0, 0) and not create_dir:
         kwargs["create_dir"] = False
 
+    if version is not None:
+        basename_template = f"{version}-{uuid4()}-{{i}}.{format.default_extname}"
+    else:
+        basename_template = f"{uuid4()}-{{i}}.{format.default_extname}"
+
     pads.write_dataset(
         arrow_table,
         schema=schema,
         base_dir=full_path,
-        basename_template=str(uuid4()) + "-{i}." + format.default_extname,
+        basename_template=basename_template,
         format=format,
         partitioning=None,
         file_options=opts,
