@@ -2,6 +2,9 @@
 use common_io_config::IOConfig;
 use serde::{Deserialize, Serialize};
 
+/// Environment variables for Daft to use when formatting displays.
+pub const BOLD_TABLE_HEADERS_IN_DISPLAY: &str = "DAFT_BOLD_TABLE_HEADERS";
+
 /// Configurations for Daft to use during the building of a Dataframe's plan.
 ///
 /// 1. Creation of a Dataframe including any file listing and schema inference that needs to happen. Note
@@ -32,16 +35,13 @@ pub struct DaftExecutionConfig {
     pub num_preview_rows: usize,
     pub parquet_target_filesize: usize,
     pub parquet_target_row_group_size: usize,
-    pub parquet_max_open_files: usize,
-    pub parquet_max_rows_per_file: usize,
-    pub parquet_min_rows_per_group: usize,
-    pub parquet_max_rows_per_group: usize,
     pub parquet_inflation_factor: f64,
     pub csv_target_filesize: usize,
     pub csv_inflation_factor: f64,
     pub shuffle_aggregation_default_partitions: usize,
     pub read_sql_partition_size_bytes: usize,
     pub enable_aqe: bool,
+    pub enable_native_executor: bool,
 }
 
 impl Default for DaftExecutionConfig {
@@ -56,16 +56,13 @@ impl Default for DaftExecutionConfig {
             num_preview_rows: 8,
             parquet_target_filesize: 512 * 1024 * 1024, // 512MB
             parquet_target_row_group_size: 128 * 1024 * 1024, // 128MB
-            parquet_max_open_files: 1024,
-            parquet_max_rows_per_file: 10 * 1024 * 1024,
-            parquet_min_rows_per_group: 64 * 1024,
-            parquet_max_rows_per_group: 128 * 1024,
             parquet_inflation_factor: 3.0,
             csv_target_filesize: 512 * 1024 * 1024, // 512MB
             csv_inflation_factor: 0.5,
             shuffle_aggregation_default_partitions: 200,
             read_sql_partition_size_bytes: 512 * 1024 * 1024, // 512MB
             enable_aqe: false,
+            enable_native_executor: false,
         }
     }
 }
@@ -79,6 +76,12 @@ impl DaftExecutionConfig {
         {
             cfg.enable_aqe = true;
         }
+        let exec_env_var_name = "DAFT_ENABLE_NATIVE_EXECUTOR";
+        if let Ok(val) = std::env::var(exec_env_var_name)
+            && matches!(val.trim().to_lowercase().as_str(), "1" | "true")
+        {
+            cfg.enable_native_executor = true;
+        }
         cfg
     }
 }
