@@ -1,9 +1,7 @@
 use std::sync::Arc;
 
-use arrow2;
-
 use crate::count_mode::CountMode;
-use crate::{array::DataArray, datatypes::*};
+use crate::datatypes::*;
 
 use common_error::DaftResult;
 
