@@ -5,6 +5,7 @@ mod arange;
 mod arithmetic;
 pub mod arrow2;
 pub mod as_arrow;
+mod between;
 pub(crate) mod broadcast;
 pub(crate) mod cast;
 mod ceil;
@@ -99,6 +100,16 @@ pub trait DaftIsIn<Rhs> {
     fn is_in(&self, rhs: Rhs) -> Self::Output;
 }
 
+pub trait DaftBetween<Lower, Upper> {
+    type Output;
+    fn between(&self, lower: Lower, upper: Upper) -> Self::Output;
+}
+
+pub trait DaftAtan2<Rhs> {
+    type Output;
+    fn atan2(&self, rhs: Rhs) -> Self::Output;
+}
+
 pub trait DaftIsNull {
     type Output;
     fn is_null(&self) -> Self::Output;
@@ -114,6 +125,16 @@ pub trait DaftIsNan {
     fn is_nan(&self) -> Self::Output;
 }
 
+pub trait DaftIsInf {
+    type Output;
+    fn is_inf(&self) -> Self::Output;
+}
+
+pub trait DaftNotNan {
+    type Output;
+    fn not_nan(&self) -> Self::Output;
+}
+
 pub type VecIndices = Vec<u64>;
 pub type GroupIndices = Vec<VecIndices>;
 pub type GroupIndicesPair = (VecIndices, GroupIndices);
