@@ -1,18 +1,20 @@
 use std::{
     borrow::Cow,
-    iter::{Repeat, Take},
+    iter::{self, Repeat, Take},
 };
 
 use crate::{
     array::{DataArray, ListArray},
     datatypes::{
-        BooleanArray, DaftIntegerType, DaftNumericType, DaftPhysicalType, Field, Int64Array,
-        UInt64Array, Utf8Array,
+        infer_timeunit_from_format_string,
+        logical::{DateArray, TimestampArray},
+        BooleanArray, DaftIntegerType, DaftNumericType, DaftPhysicalType, Field, Int32Array,
+        Int64Array, TimeUnit, UInt64Array, Utf8Array,
     },
     DataType, Series,
 };
-use arrow2::array::Array;
-
+use arrow2::{array::Array, temporal_conversions};
+use chrono::Datelike;
 use common_error::{DaftError, DaftResult};
 use itertools::Itertools;
 use num_traits::NumCast;
@@ -271,6 +273,75 @@ fn replace_on_literal<'a>(
     Ok(Utf8Array::from((name, Box::new(arrow_result?))))
 }
 
+fn substring(s: &str, start: usize, len: Option<usize>) -> Option<&str> {
+    let mut char_indices = s.char_indices();
+
+    if let Some((start_pos, _)) = char_indices.nth(start) {
+        let len = match len {
+            Some(len) => {
+                if len == 0 {
+                    return None;
+                } else {
+                    len
+                }
+            }
+            None => {
+                return Some(&s[start_pos..]);
+            }
+        };
+
+        let end_pos = char_indices
+            .nth(len.saturating_sub(1))
+            .map_or(s.len(), |(idx, _)| idx);
+
+        Some(&s[start_pos..end_pos])
+    } else {
+        None
+    }
+}
+
+fn substr_compute_result<I, U, E, R>(
+    name: &str,
+    iter: BroadcastedStrIter,
+    start: I,
+    length: U,
+) -> DaftResult<Utf8Array>
+where
+    I: Iterator<Item = Result<Option<usize>, E>>,
+    U: Iterator<Item = Result<Option<usize>, R>>,
+{
+    let arrow_result = iter
+        .zip(start)
+        .zip(length)
+        .map(|((val, s), l)| {
+            let s = match s {
+                Ok(s) => s,
+                Err(_) => {
+                    return Err(DaftError::ComputeError(
+                        "Error in repeat: failed to cast length as usize".to_string(),
+                    ))
+                }
+            };
+            let l = match l {
+                Ok(l) => l,
+                Err(_) => {
+                    return Err(DaftError::ComputeError(
+                        "Error in repeat: failed to cast length as usize".to_string(),
+                    ))
+                }
+            };
+
+            match (val, s, l) {
+                (Some(val), Some(s), Some(l)) => Ok(substring(val, s, Some(l))),
+                (Some(val), Some(s), None) => Ok(substring(val, s, None)),
+                _ => Ok(None),
+            }
+        })
+        .collect::<DaftResult<arrow2::array::Utf8Array<i64>>>()?;
+
+    Ok(Utf8Array::from((name, Box::new(arrow_result))))
+}
+
 #[derive(Debug, Clone, Copy)]
 pub enum PadPlacement {
     Left,
@@ -822,6 +893,91 @@ impl Utf8Array {
         Ok(result)
     }
 
+    pub fn to_date(&self, format: &str) -> DaftResult<DateArray> {
+        let len = self.len();
+        let self_iter = self.as_arrow().iter();
+
+        let arrow_result = self_iter
+            .map(|val| match val {
+                Some(val) => {
+                    let date = chrono::NaiveDate::parse_from_str(val, format).map_err(|e| {
+                        DaftError::ComputeError(format!(
+                            "Error in to_date: failed to parse date {val} with format {format} : {e}"
+                        ))
+                    })?;
+                    Ok(Some(
+                        date.num_days_from_ce() - temporal_conversions::EPOCH_DAYS_FROM_CE,
+                    ))
+                }
+                _ => Ok(None),
+            })
+            .collect::<DaftResult<arrow2::array::Int32Array>>()?;
+
+        let result = Int32Array::from((self.name(), Box::new(arrow_result)));
+        let result = DateArray::new(Field::new(self.name(), DataType::Date), result);
+        assert_eq!(result.len(), len);
+        Ok(result)
+    }
+
+    pub fn to_datetime(&self, format: &str, timezone: Option<&str>) -> DaftResult<TimestampArray> {
+        let len = self.len();
+        let self_iter = self.as_arrow().iter();
+        let timeunit = infer_timeunit_from_format_string(format);
+
+        let arrow_result = self_iter
+            .map(|val| match val {
+                Some(val) => {
+                    let timestamp = match timezone {
+                        Some(tz) => {
+                            let datetime = chrono::DateTime::parse_from_str(val, format).map_err(|e| {
+                                DaftError::ComputeError(format!(
+                                    "Error in to_datetime: failed to parse datetime {val} with format {format} : {e}"
+                                ))
+                            })?;
+                            let datetime_with_timezone = datetime.with_timezone(&tz.parse::<chrono_tz::Tz>().map_err(|e| {
+                                DaftError::ComputeError(format!(
+                                    "Error in to_datetime: failed to parse timezone {tz} : {e}"
+                                ))
+                            })?);
+                            match timeunit {
+                                TimeUnit::Seconds => datetime_with_timezone.timestamp(),
+                                TimeUnit::Milliseconds => datetime_with_timezone.timestamp_millis(),
+                                TimeUnit::Microseconds => datetime_with_timezone.timestamp_micros(),
+                                TimeUnit::Nanoseconds => datetime_with_timezone.timestamp_nanos_opt().ok_or_else(|| DaftError::ComputeError(format!("Error in to_datetime: failed to get nanoseconds for {val}")))?,
+                            }
+                        }
+                        None => {
+                            let naive_datetime = chrono::NaiveDateTime::parse_from_str(val, format).map_err(|e| {
+                                DaftError::ComputeError(format!(
+                                    "Error in to_datetime: failed to parse datetime {val} with format {format} : {e}"
+                                ))
+                            })?;
+                            match timeunit {
+                                TimeUnit::Seconds => naive_datetime.and_utc().timestamp(),
+                                TimeUnit::Milliseconds => naive_datetime.and_utc().timestamp_millis(),
+                                TimeUnit::Microseconds => naive_datetime.and_utc().timestamp_micros(),
+                                TimeUnit::Nanoseconds => naive_datetime.and_utc().timestamp_nanos_opt().ok_or_else(|| DaftError::ComputeError(format!("Error in to_datetime: failed to get nanoseconds for {val}")))?,
+                            }
+                        }
+                    };
+                    Ok(Some(timestamp))
+                }
+                _ => Ok(None),
+            })
+            .collect::<DaftResult<arrow2::array::Int64Array>>()?;
+
+        let result = Int64Array::from((self.name(), Box::new(arrow_result)));
+        let result = TimestampArray::new(
+            Field::new(
+                self.name(),
+                DataType::Timestamp(timeunit, timezone.map(|tz| tz.to_string())),
+            ),
+            result,
+        );
+        assert_eq!(result.len(), len);
+        Ok(result)
+    }
+
     pub fn repeat<I>(&self, n: &DataArray<I>) -> DaftResult<Utf8Array>
     where
         I: DaftIntegerType,
@@ -879,6 +1035,125 @@ impl Utf8Array {
         Ok(result)
     }
 
+    pub fn substr<I, J>(
+        &self,
+        start: &DataArray<I>,
+        length: Option<&DataArray<J>>,
+    ) -> DaftResult<Utf8Array>
+    where
+        I: DaftIntegerType,
+        <I as DaftNumericType>::Native: Ord,
+        J: DaftIntegerType,
+        <J as DaftNumericType>::Native: Ord,
+    {
+        let name = self.name();
+        let (is_full_null, expected_size) = parse_inputs(self, &[start])
+            .map_err(|e| DaftError::ValueError(format!("Error in substr: {e}")))?;
+
+        if is_full_null {
+            return Ok(Utf8Array::full_null(name, &DataType::Utf8, expected_size));
+        }
+
+        let self_iter = create_broadcasted_str_iter(self, expected_size);
+
+        let (length_repeat, length_iter) = match length {
+            Some(length) => {
+                if length.len() != 1 && length.len() != expected_size {
+                    return Err(DaftError::ValueError(
+                        "Inputs have invalid lengths: length".to_string(),
+                    ));
+                }
+
+                match length.len() {
+                    1 => {
+                        let length_repeat: Result<Option<usize>, ()> = if length.null_count() == 1 {
+                            Ok(None)
+                        } else {
+                            let val = length.get(0).unwrap();
+                            let val: usize = NumCast::from(val).ok_or_else(|| {
+                                DaftError::ComputeError(format!(
+                                    "Error in substr: failed to cast length as usize {val}"
+                                ))
+                            })?;
+
+                            Ok(Some(val))
+                        };
+
+                        let length_repeat = iter::repeat(length_repeat).take(expected_size);
+                        (Some(length_repeat), None)
+                    }
+                    _ => {
+                        let length_iter = length.as_arrow().iter().map(|l| match l {
+                            Some(l) => {
+                                let l: usize = NumCast::from(*l).ok_or_else(|| {
+                                    DaftError::ComputeError(format!(
+                                        "Error in repeat: failed to cast length as usize {l}"
+                                    ))
+                                })?;
+                                let result: Result<Option<usize>, DaftError> = Ok(Some(l));
+                                result
+                            }
+                            None => Ok(None),
+                        });
+                        (None, Some(length_iter))
+                    }
+                }
+            }
+            None => {
+                let none_value_iter = iter::repeat(Ok(None)).take(expected_size);
+                (Some(none_value_iter), None)
+            }
+        };
+
+        let (start_repeat, start_iter) = match start.len() {
+            1 => {
+                let start_repeat = start.get(0).unwrap();
+                let start_repeat: usize = NumCast::from(start_repeat).ok_or_else(|| {
+                    DaftError::ComputeError(format!(
+                        "Error in substr: failed to cast start as usize {start_repeat}"
+                    ))
+                })?;
+                let start_repeat: Result<Option<usize>, ()> = Ok(Some(start_repeat));
+                let start_repeat = iter::repeat(start_repeat).take(expected_size);
+                (Some(start_repeat), None)
+            }
+            _ => {
+                let start_iter = start.as_arrow().iter().map(|s| match s {
+                    Some(s) => {
+                        let s: usize = NumCast::from(*s).ok_or_else(|| {
+                            DaftError::ComputeError(format!(
+                                "Error in repeat: failed to cast length as usize {s}"
+                            ))
+                        })?;
+                        let result: Result<Option<usize>, DaftError> = Ok(Some(s));
+                        result
+                    }
+                    None => Ok(None),
+                });
+                (None, Some(start_iter))
+            }
+        };
+
+        match (start_iter, start_repeat, length_iter, length_repeat) {
+            (Some(start_iter), None, Some(length_iter), None) => {
+                substr_compute_result(name, self_iter, start_iter, length_iter)
+            }
+            (Some(start_iter), None, None, Some(length_repeat)) => {
+                substr_compute_result(name, self_iter, start_iter, length_repeat)
+            }
+            (None, Some(start_repeat), Some(length_iter), None) => {
+                substr_compute_result(name, self_iter, start_repeat, length_iter)
+            }
+            (None, Some(start_repeat), None, Some(length_repeat)) => {
+                substr_compute_result(name, self_iter, start_repeat, length_repeat)
+            }
+
+            _ => Err(DaftError::ComputeError(
+                "Start and length parameters are empty".to_string(),
+            )),
+        }
+    }
+
     pub fn pad<I>(
         &self,
         length: &DataArray<I>,
