@@ -183,6 +183,10 @@ impl PySeries {
             .into())
     }
 
+    pub fn arctan2(&self, other: &Self) -> PyResult<Self> {
+        Ok(self.series.atan2(&other.series)?.into())
+    }
+
     pub fn degrees(&self) -> PyResult<Self> {
         Ok(self
             .series
@@ -205,6 +209,10 @@ impl PySeries {
         Ok(self.series.log10()?.into())
     }
 
+    pub fn log(&self, base: f64) -> PyResult<Self> {
+        Ok(self.series.log(base)?.into())
+    }
+
     pub fn ln(&self) -> PyResult<Self> {
         Ok(self.series.ln()?.into())
     }
@@ -446,10 +454,37 @@ impl PySeries {
         Ok(self.series.utf8_ilike(&pattern.series)?.into())
     }
 
+    pub fn utf8_substr(&self, start: &Self, length: &Self) -> PyResult<Self> {
+        Ok(self
+            .series
+            .utf8_substr(&start.series, &length.series)?
+            .into())
+    }
+
+    pub fn utf8_to_date(&self, format: &str) -> PyResult<Self> {
+        Ok(self.series.utf8_to_date(format)?.into())
+    }
+
+    pub fn utf8_to_datetime(&self, format: &str, timezone: Option<&str>) -> PyResult<Self> {
+        Ok(self.series.utf8_to_datetime(format, timezone)?.into())
+    }
+
     pub fn is_nan(&self) -> PyResult<Self> {
         Ok(self.series.is_nan()?.into())
     }
 
+    pub fn is_inf(&self) -> PyResult<Self> {
+        Ok(self.series.is_inf()?.into())
+    }
+
+    pub fn not_nan(&self) -> PyResult<Self> {
+        Ok(self.series.not_nan()?.into())
+    }
+
+    pub fn fill_nan(&self, fill_value: &Self) -> PyResult<Self> {
+        Ok(self.series.fill_nan(&fill_value.series)?.into())
+    }
+
     pub fn dt_date(&self) -> PyResult<Self> {
         Ok(self.series.dt_date()?.into())
     }
