@@ -2,6 +2,7 @@ use crate::array::ListArray;
 use crate::count_mode::CountMode;
 use crate::series::IntoSeries;
 use crate::{array::ops::GroupIndices, series::Series, with_match_physical_daft_types};
+use arrow2::array::PrimitiveArray;
 use common_error::{DaftError, DaftResult};
 
 use crate::datatypes::*;
@@ -67,13 +68,59 @@ impl Series {
         }
     }
 
+    pub fn approx_sketch(&self, groups: Option<&GroupIndices>) -> DaftResult<Series> {
+        use crate::array::ops::DaftApproxSketchAggable;
+        use crate::datatypes::DataType::*;
+
+        // Upcast all numeric types to float64 and compute approx_sketch.
+        match self.data_type() {
+            dt if dt.is_numeric() => {
+                let casted = self.cast(&Float64)?;
+                match groups {
+                    Some(groups) => Ok(DaftApproxSketchAggable::grouped_approx_sketch(
+                        &casted.f64()?,
+                        groups,
+                    )?
+                    .into_series()),
+                    None => {
+                        Ok(DaftApproxSketchAggable::approx_sketch(&casted.f64()?)?.into_series())
+                    }
+                }
+            }
+            other => Err(DaftError::TypeError(format!(
+                "Approx sketch is not implemented for type {}",
+                other
+            ))),
+        }
+    }
+
+    pub fn merge_sketch(&self, groups: Option<&GroupIndices>) -> DaftResult<Series> {
+        use crate::array::ops::DaftMergeSketchAggable;
+        use crate::datatypes::DataType::*;
+
+        match self.data_type() {
+            Struct(_) => match groups {
+                Some(groups) => Ok(DaftMergeSketchAggable::grouped_merge_sketch(
+                    &self.struct_()?,
+                    groups,
+                )?
+                .into_series()),
+                None => Ok(DaftMergeSketchAggable::merge_sketch(&self.struct_()?)?.into_series()),
+            },
+            other => Err(DaftError::TypeError(format!(
+                "Merge sketch is not implemented for type {}",
+                other
+            ))),
+        }
+    }
+
     pub fn mean(&self, groups: Option<&GroupIndices>) -> DaftResult<Series> {
         use crate::array::ops::DaftMeanAggable;
         use crate::datatypes::DataType::*;
 
         // Upcast all numeric types to float64 and use f64 mean kernel.
         match self.data_type() {
-            Int8 | Int16 | Int32 | Int64 | UInt8 | UInt16 | UInt32 | UInt64 | Float32 | Float64 => {
+            dt if dt.is_numeric() => {
                 let casted = self.cast(&Float64)?;
                 match groups {
                     Some(groups) => {
@@ -97,6 +144,47 @@ impl Series {
         self.inner.max(groups)
     }
 
+    pub fn any_value(
+        &self,
+        groups: Option<&GroupIndices>,
+        ignore_nulls: bool,
+    ) -> DaftResult<Series> {
+        let indices = match groups {
+            Some(groups) => {
+                if self.data_type().is_null() {
+                    Box::new(PrimitiveArray::new_null(
+                        arrow2::datatypes::DataType::UInt64,
+                        groups.len(),
+                    ))
+                } else if ignore_nulls && let Some(validity) = self.validity() {
+                    Box::new(PrimitiveArray::from_trusted_len_iter(groups.iter().map(
+                        |g| g.iter().find(|i| validity.get_bit(**i as usize)).copied(),
+                    )))
+                } else {
+                    Box::new(PrimitiveArray::from_trusted_len_iter(
+                        groups.iter().map(|g| g.first().cloned()),
+                    ))
+                }
+            }
+            None => {
+                let idx = if self.data_type().is_null() || self.is_empty() {
+                    None
+                } else if ignore_nulls && let Some(validity) = self.validity() {
+                    validity.iter().position(|v| v).map(|i| i as u64)
+                } else {
+                    Some(0)
+                };
+
+                Box::new(PrimitiveArray::from([idx]))
+            }
+        };
+
+        self.take(&Series::from_arrow(
+            Field::new("", DataType::UInt64).into(),
+            indices,
+        )?)
+    }
+
     pub fn agg_list(&self, groups: Option<&GroupIndices>) -> DaftResult<Series> {
         self.inner.agg_list(groups)
     }
