@@ -7,11 +7,13 @@ use crate::{
 use common_error::DaftResult;
 
 use super::Series;
-pub trait SeriesLike: Send + Sync + Any {
+pub trait SeriesLike: Send + Sync + Any + std::fmt::Debug {
     #[allow(clippy::wrong_self_convention)]
     fn into_series(&self) -> Series;
     fn to_arrow(&self) -> Box<dyn arrow2::array::Array>;
     fn as_any(&self) -> &dyn std::any::Any;
+    fn with_validity(&self, validity: Option<arrow2::bitmap::Bitmap>) -> DaftResult<Series>;
+    fn validity(&self) -> Option<&arrow2::bitmap::Bitmap>;
     fn min(&self, groups: Option<&GroupIndices>) -> DaftResult<Series>;
     fn max(&self, groups: Option<&GroupIndices>) -> DaftResult<Series>;
     fn agg_list(&self, groups: Option<&GroupIndices>) -> DaftResult<Series>;
@@ -26,6 +28,7 @@ pub trait SeriesLike: Send + Sync + Any {
     fn rename(&self, name: &str) -> Series;
     fn size_bytes(&self) -> DaftResult<usize>;
     fn is_null(&self) -> DaftResult<Series>;
+    fn not_null(&self) -> DaftResult<Series>;
     fn sort(&self, descending: bool) -> DaftResult<Series>;
     fn head(&self, num: usize) -> DaftResult<Series>;
     fn slice(&self, start: usize, end: usize) -> DaftResult<Series>;
