@@ -356,7 +356,7 @@ async fn read_csv_single_into_stream(
                         .unwrap_or(64 * 1024),
                 )
             }
-            GetResult::Stream(stream, _, _) => (
+            GetResult::Stream(stream, ..) => (
                 Box::new(StreamReader::new(stream)),
                 read_options
                     .as_ref()
@@ -380,6 +380,7 @@ async fn read_csv_single_into_stream(
         .escape(parse_options.escape_char)
         .comment(parse_options.comment)
         .buffer_capacity(buffer_size)
+        .flexible(parse_options.allow_variable_columns)
         .create_reader(reader.compat());
     let read_stream = read_into_byterecord_chunk_stream(
         reader,
@@ -1541,6 +1542,55 @@ mod tests {
         Ok(())
     }
 
+    #[test]
+    fn test_csv_read_local_invalid_cols_header_mismatch_allow_variable_columns() -> DaftResult<()> {
+        let file = format!(
+            "{}/test/iris_tiny_invalid_header_cols_mismatch.csv", // 5 cols in header with 4 cols in data
+            env!("CARGO_MANIFEST_DIR"),
+        );
+
+        let mut io_config = IOConfig::default();
+        io_config.s3.anonymous = true;
+
+        let io_client = Arc::new(IOClient::new(io_config.into())?);
+
+        let table = read_csv(
+            file.as_ref(),
+            None,
+            Some(CsvParseOptions::default().with_variable_columns(true)),
+            None,
+            io_client,
+            None,
+            true,
+            None,
+        )?;
+
+        assert_eq!(table.len(), 3);
+
+        assert_eq!(
+            table.schema,
+            Schema::new(vec![
+                Field::new("sepal.length", DataType::Float64),
+                Field::new("sepal.width", DataType::Float64),
+                Field::new("petal.length", DataType::Float64),
+                Field::new("petal.width", DataType::Float64),
+                Field::new("variety", DataType::Utf8),
+            ])?
+            .into(),
+        );
+
+        // First 4 cols should have no nulls
+        assert_eq!(table.get_column("sepal.length")?.to_arrow().null_count(), 0);
+        assert_eq!(table.get_column("sepal.width")?.to_arrow().null_count(), 0);
+        assert_eq!(table.get_column("petal.length")?.to_arrow().null_count(), 0);
+        assert_eq!(table.get_column("petal.width")?.to_arrow().null_count(), 0);
+
+        // Last col should have 3 nulls because of the missing data
+        assert_eq!(table.get_column("variety")?.to_arrow().null_count(), 3);
+
+        Ok(())
+    }
+
     #[test]
     fn test_csv_read_local_invalid_no_header_variable_num_cols() -> DaftResult<()> {
         let file = format!(
@@ -1576,6 +1626,98 @@ mod tests {
         Ok(())
     }
 
+    #[test]
+    fn test_csv_read_local_invalid_no_header_allow_variable_cols() -> DaftResult<()> {
+        let file = format!(
+            "{}/test/iris_tiny_invalid_no_header_variable_num_cols.csv", // first and third row have 4 cols, second row has 5 cols
+            env!("CARGO_MANIFEST_DIR"),
+        );
+
+        let mut io_config = IOConfig::default();
+        io_config.s3.anonymous = true;
+
+        let io_client = Arc::new(IOClient::new(io_config.into())?);
+
+        let table = read_csv(
+            file.as_ref(),
+            None,
+            Some(
+                CsvParseOptions::default()
+                    .with_has_header(false)
+                    .with_variable_columns(true),
+            ),
+            None,
+            io_client,
+            None,
+            true,
+            None,
+        )?;
+
+        assert_eq!(table.len(), 3);
+
+        assert_eq!(
+            table.schema,
+            Schema::new(vec![
+                Field::new("column_1", DataType::Float64),
+                Field::new("column_2", DataType::Float64),
+                Field::new("column_3", DataType::Float64),
+                Field::new("column_4", DataType::Float64),
+            ])?
+            .into(),
+        );
+
+        Ok(())
+    }
+
+    #[test]
+    fn test_csv_read_local_invalid_no_header_allow_variable_cols_with_schema() -> DaftResult<()> {
+        let file = format!(
+            "{}/test/iris_tiny_invalid_no_header_variable_num_cols.csv", // first and third row have 4 cols, second row has 5 cols
+            env!("CARGO_MANIFEST_DIR"),
+        );
+
+        let mut io_config = IOConfig::default();
+        io_config.s3.anonymous = true;
+
+        let io_client = Arc::new(IOClient::new(io_config.into())?);
+
+        let schema = Schema::new(vec![
+            Field::new("sepal.length", DataType::Float64),
+            Field::new("sepal.width", DataType::Float64),
+            Field::new("petal.length", DataType::Float64),
+            Field::new("petal.width", DataType::Float64),
+            Field::new("variety", DataType::Utf8),
+        ])?;
+
+        let table = read_csv(
+            file.as_ref(),
+            Some(CsvConvertOptions::default().with_schema(Some(schema.into()))),
+            Some(
+                CsvParseOptions::default()
+                    .with_has_header(false)
+                    .with_variable_columns(true),
+            ),
+            None,
+            io_client,
+            None,
+            true,
+            None,
+        )?;
+
+        assert_eq!(table.len(), 3);
+
+        assert_eq!(
+            table.get_column("variety")?.to_arrow(),
+            Box::new(arrow2::array::Utf8Array::<i64>::from(vec![
+                None,
+                Some("Seratosa"),
+                None,
+            ])) as Box<dyn arrow2::array::Array>
+        );
+
+        Ok(())
+    }
+
     #[rstest]
     fn test_csv_read_s3_compression(
         #[values(
