@@ -4,11 +4,11 @@ use daft_core::{
     series::Series,
 };
 
-use crate::Expr;
+use crate::{functions::FunctionExpr, ExprRef};
 
 use common_error::{DaftError, DaftResult};
 
-use super::super::FunctionEvaluator;
+use super::{super::FunctionEvaluator, ImageExpr};
 
 pub struct DecodeEvaluator {}
 
@@ -17,7 +17,7 @@ impl FunctionEvaluator for DecodeEvaluator {
         "decode"
     }
 
-    fn to_field(&self, inputs: &[Expr], schema: &Schema, _: &Expr) -> DaftResult<Field> {
+    fn to_field(&self, inputs: &[ExprRef], schema: &Schema, _: &FunctionExpr) -> DaftResult<Field> {
         match inputs {
             [input] => {
                 let field = input.to_field(schema)?;
@@ -36,9 +36,15 @@ impl FunctionEvaluator for DecodeEvaluator {
         }
     }
 
-    fn evaluate(&self, inputs: &[Series], _: &Expr) -> DaftResult<Series> {
+    fn evaluate(&self, inputs: &[Series], expr: &FunctionExpr) -> DaftResult<Series> {
+        let raise_error_on_failure = match expr {
+            FunctionExpr::Image(ImageExpr::Decode {
+                raise_error_on_failure,
+            }) => raise_error_on_failure,
+            _ => panic!("DecodeEvaluator expects an Image::Decode expression"),
+        };
         match inputs {
-            [input] => input.image_decode(),
+            [input] => input.image_decode(*raise_error_on_failure),
             _ => Err(DaftError::ValueError(format!(
                 "Expected 1 input arg, got {}",
                 inputs.len()
