@@ -5,7 +5,7 @@ use common_error::DaftResult;
 use daft_core::datatypes::DataType;
 use serde::{Deserialize, Serialize};
 
-use crate::Expr;
+use crate::{Expr, ExprRef};
 
 #[derive(Debug, Clone, Serialize, Deserialize, PartialEq, Eq, Hash)]
 pub struct PythonUDF {
@@ -14,7 +14,11 @@ pub struct PythonUDF {
     return_dtype: DataType,
 }
 
-pub fn udf(func: pyo3::PyObject, expressions: &[Expr], return_dtype: DataType) -> DaftResult<Expr> {
+pub fn udf(
+    func: pyo3::PyObject,
+    expressions: &[ExprRef],
+    return_dtype: DataType,
+) -> DaftResult<Expr> {
     Ok(Expr::Function {
         func: super::FunctionExpr::Python(PythonUDF {
             func: partial_udf::PartialUDF(func),
