@@ -4,7 +4,8 @@ use daft_core::{
     series::Series,
 };
 
-use crate::Expr;
+use crate::functions::FunctionExpr;
+use crate::ExprRef;
 use common_error::{DaftError, DaftResult};
 
 use super::super::FunctionEvaluator;
@@ -16,7 +17,7 @@ impl FunctionEvaluator for LengthEvaluator {
         "length"
     }
 
-    fn to_field(&self, inputs: &[Expr], schema: &Schema, _: &Expr) -> DaftResult<Field> {
+    fn to_field(&self, inputs: &[ExprRef], schema: &Schema, _: &FunctionExpr) -> DaftResult<Field> {
         match inputs {
             [data] => match data.to_field(schema) {
                 Ok(data_field) => match &data_field.dtype {
@@ -34,7 +35,7 @@ impl FunctionEvaluator for LengthEvaluator {
         }
     }
 
-    fn evaluate(&self, inputs: &[Series], _: &Expr) -> DaftResult<Series> {
+    fn evaluate(&self, inputs: &[Series], _: &FunctionExpr) -> DaftResult<Series> {
         match inputs {
             [data] => data.utf8_length(),
             _ => Err(DaftError::ValueError(format!(
