@@ -20,6 +20,9 @@ mod rpad;
 mod rstrip;
 mod split;
 mod startswith;
+mod substr;
+mod to_date;
+mod to_datetime;
 mod upper;
 
 use capitalize::CapitalizeEvaluator;
@@ -44,6 +47,9 @@ use rstrip::RstripEvaluator;
 use serde::{Deserialize, Serialize};
 use split::SplitEvaluator;
 use startswith::StartswithEvaluator;
+use substr::SubstrEvaluator;
+use to_date::ToDateEvaluator;
+use to_datetime::ToDatetimeEvaluator;
 use upper::UpperEvaluator;
 
 use crate::{functions::utf8::match_::MatchEvaluator, Expr, ExprRef};
@@ -75,6 +81,9 @@ pub enum Utf8Expr {
     Repeat,
     Like,
     Ilike,
+    Substr,
+    ToDate(String),
+    ToDatetime(String, Option<String>),
 }
 
 impl Utf8Expr {
@@ -105,6 +114,9 @@ impl Utf8Expr {
             Repeat => &RepeatEvaluator {},
             Like => &LikeEvaluator {},
             Ilike => &IlikeEvaluator {},
+            Substr => &SubstrEvaluator {},
+            ToDate(_) => &ToDateEvaluator {},
+            ToDatetime(_, _) => &ToDatetimeEvaluator {},
         }
     }
 }
@@ -292,3 +304,30 @@ pub fn ilike(data: ExprRef, pattern: ExprRef) -> ExprRef {
     }
     .into()
 }
+
+pub fn substr(data: ExprRef, start: ExprRef, length: ExprRef) -> ExprRef {
+    Expr::Function {
+        func: super::FunctionExpr::Utf8(Utf8Expr::Substr),
+        inputs: vec![data, start, length],
+    }
+    .into()
+}
+
+pub fn to_date(data: ExprRef, format: &str) -> ExprRef {
+    Expr::Function {
+        func: super::FunctionExpr::Utf8(Utf8Expr::ToDate(format.to_string())),
+        inputs: vec![data],
+    }
+    .into()
+}
+
+pub fn to_datetime(data: ExprRef, format: &str, timezone: Option<&str>) -> ExprRef {
+    Expr::Function {
+        func: super::FunctionExpr::Utf8(Utf8Expr::ToDatetime(
+            format.to_string(),
+            timezone.map(|s| s.to_string()),
+        )),
+        inputs: vec![data],
+    }
+    .into()
+}
