@@ -260,6 +260,11 @@ impl PyExpr {
         Ok(arctan(self.into()).into())
     }
 
+    pub fn arctan2(&self, other: &Self) -> PyResult<Self> {
+        use functions::numeric::arctan2;
+        Ok(arctan2(self.into(), other.expr.clone()).into())
+    }
+
     pub fn radians(&self) -> PyResult<Self> {
         use functions::numeric::radians;
         Ok(radians(self.into()).into())
@@ -280,6 +285,11 @@ impl PyExpr {
         Ok(log10(self.into()).into())
     }
 
+    pub fn log(&self, base: f64) -> PyResult<Self> {
+        use functions::numeric::log;
+        Ok(log(self.into(), base).into())
+    }
+
     pub fn ln(&self) -> PyResult<Self> {
         use functions::numeric::ln;
         Ok(ln(self.into()).into())
@@ -435,6 +445,14 @@ impl PyExpr {
         Ok(self.expr.clone().is_in(other.expr.clone()).into())
     }
 
+    pub fn between(&self, lower: &Self, upper: &Self) -> PyResult<Self> {
+        Ok(self
+            .expr
+            .clone()
+            .between(lower.expr.clone(), upper.expr.clone())
+            .into())
+    }
+
     pub fn name(&self) -> PyResult<&str> {
         Ok(self.expr.name())
     }
@@ -462,6 +480,21 @@ impl PyExpr {
         Ok(is_nan(self.into()).into())
     }
 
+    pub fn is_inf(&self) -> PyResult<Self> {
+        use functions::float::is_inf;
+        Ok(is_inf(self.into()).into())
+    }
+
+    pub fn not_nan(&self) -> PyResult<Self> {
+        use functions::float::not_nan;
+        Ok(not_nan(self.into()).into())
+    }
+
+    pub fn fill_nan(&self, fill_value: &Self) -> PyResult<Self> {
+        use functions::float::fill_nan;
+        Ok(fill_nan(self.into(), fill_value.expr.clone()).into())
+    }
+
     pub fn dt_date(&self) -> PyResult<Self> {
         use functions::temporal::date;
         Ok(date(self.into()).into())
@@ -633,6 +666,21 @@ impl PyExpr {
         Ok(ilike(self.into(), pattern.into()).into())
     }
 
+    pub fn utf8_substr(&self, start: &Self, length: &Self) -> PyResult<Self> {
+        use crate::functions::utf8::substr;
+        Ok(substr(self.into(), start.into(), length.into()).into())
+    }
+
+    pub fn utf8_to_date(&self, format: &str) -> PyResult<Self> {
+        use crate::functions::utf8::to_date;
+        Ok(to_date(self.into(), format).into())
+    }
+
+    pub fn utf8_to_datetime(&self, format: &str, timezone: Option<&str>) -> PyResult<Self> {
+        use crate::functions::utf8::to_datetime;
+        Ok(to_datetime(self.into(), format, timezone).into())
+    }
+
     pub fn image_decode(&self, raise_error_on_failure: bool) -> PyResult<Self> {
         use crate::functions::image::decode;
         Ok(decode(self.into(), raise_error_on_failure).into())
