@@ -1,4 +1,6 @@
 use async_trait::async_trait;
+use azure_core::{auth::TokenCredential, new_http_client};
+use azure_identity::{ClientSecretCredential, DefaultAzureCredential};
 use azure_storage::{prelude::*, CloudLocation};
 use azure_storage_blobs::{
     blob::operations::GetBlobResponse,
@@ -19,6 +21,8 @@ use common_io_config::AzureConfig;
 
 const AZURE_DELIMITER: &str = "/";
 const DEFAULT_GLOB_FANOUT_LIMIT: usize = 1024;
+const AZURE_STORAGE_RESOURCE: &str = "https://storage.azure.com";
+const AZURE_STORE_SUFFIX: &str = ".dfs.core.windows.net";
 
 #[derive(Debug, Snafu)]
 enum Error {
@@ -61,6 +65,44 @@ enum Error {
     NotAFile { path: String },
 }
 
+/// Parse an Azure URI into its components.
+/// Returns (protocol, container if exists, key).
+fn parse_azure_uri(uri: &str) -> super::Result<(String, Option<(String, String)>)> {
+    let uri = url::Url::parse(uri).with_context(|_| InvalidUrlSnafu { path: uri })?;
+
+    // "Container" is Azure's name for Bucket.
+    //
+    // fsspec supports two URI formats; for compatibility, we will support both as well.
+    // PROTOCOL://container/path-part/file
+    // PROTOCOL://container@account.dfs.core.windows.net/path-part/file
+    // See https://github.com/fsspec/adlfs/ for more details
+    //
+    // It also supports PROTOCOL://account.dfs.core.windows.net/container/path-part/file
+    // but it is not documented
+    // https://github.com/fsspec/adlfs/blob/5c24b2e886fc8e068a313819ce3db9b7077c27e3/adlfs/spec.py#L364
+    let username = uri.username();
+    let container_and_key = if username.is_empty() {
+        match uri.host_str() {
+            Some(host) if host.ends_with(AZURE_STORE_SUFFIX) => uri
+                .path()
+                .split_once('/')
+                .map(|(c, k)| (c.into(), k.into())),
+            Some(host) => Some((host.into(), uri.path().into())),
+            None => None,
+        }
+    } else {
+        Some((username.into(), uri.path().into()))
+    };
+
+    // fsspec supports multiple URI protocol strings for Azure: az:// and abfs://.
+    // NB: It's unclear if there is a semantic difference between the protocols
+    // or if there is a standard for the behaviour either;
+    // here, we will treat them both the same, but persist whichever protocol string was used.
+    let protocol = uri.scheme();
+
+    Ok((protocol.into(), container_and_key))
+}
+
 impl From<Error> for super::Error {
     fn from(error: Error) -> Self {
         use Error::*;
@@ -109,15 +151,47 @@ impl AzureBlobSource {
             // TODO(Clark): Allow no storage account + anonymous access + custom endpoint.
             return Err(Error::StorageAccountNotSet.into());
         };
+
+        let access_key = config
+            .access_key
+            .clone()
+            .or_else(|| std::env::var("AZURE_STORAGE_KEY").ok());
+        let sas_token = config
+            .sas_token
+            .clone()
+            .or_else(|| std::env::var("AZURE_STORAGE_SAS_TOKEN").ok());
+
         let storage_credentials = if config.anonymous {
             StorageCredentials::anonymous()
-        } else if let Some(access_key) = &config.access_key {
-            StorageCredentials::access_key(&storage_account, access_key)
-        } else if let Ok(access_key) = std::env::var("AZURE_STORAGE_KEY") {
+        } else if let Some(access_key) = access_key {
             StorageCredentials::access_key(&storage_account, access_key)
+        } else if let Some(sas_token) = sas_token {
+            StorageCredentials::sas_token(sas_token)
+                .map_err(|e| Error::AzureGeneric { source: e })?
+        } else if let Some(tenant_id) = &config.tenant_id
+            && let Some(client_id) = &config.client_id
+            && let Some(client_secret) = &config.client_secret
+        {
+            StorageCredentials::token_credential(Arc::new(ClientSecretCredential::new(
+                new_http_client(),
+                tenant_id.clone(),
+                client_id.clone(),
+                client_secret.clone(),
+                Default::default(),
+            )))
         } else {
-            log::warn!("Azure access key not found, Set either `AzureConfig.access_key` or the `AZURE_STORAGE_KEY` environment variable. Defaulting to anonymous mode.");
-            StorageCredentials::anonymous()
+            let default_creds = Arc::new(DefaultAzureCredential::default());
+
+            if default_creds
+                .get_token(AZURE_STORAGE_RESOURCE)
+                .await
+                .is_ok()
+            {
+                StorageCredentials::token_credential(default_creds)
+            } else {
+                log::warn!("Azure credentials resolution failed. Defaulting to anonymous mode.");
+                StorageCredentials::anonymous()
+            }
         };
         let endpoint_url = if let Some(endpoint_url) = &config.endpoint_url {
             Some(endpoint_url.clone())
@@ -408,15 +482,11 @@ impl ObjectSource for AzureBlobSource {
         range: Option<Range<usize>>,
         io_stats: Option<IOStatsRef>,
     ) -> super::Result<GetResult> {
-        let parsed = url::Url::parse(uri).with_context(|_| InvalidUrlSnafu { path: uri })?;
-        let container = match parsed.host_str() {
-            Some(s) => Ok(s),
-            None => Err(Error::InvalidUrl {
-                path: uri.into(),
-                source: url::ParseError::EmptyHost,
-            }),
-        }?;
-        let key = parsed.path();
+        let (_, container_and_key) = parse_azure_uri(uri)?;
+        let (container, key) = container_and_key.ok_or_else(|| Error::InvalidUrl {
+            path: uri.into(),
+            source: url::ParseError::EmptyHost,
+        })?;
 
         if key.is_empty() {
             return Err(Error::NotAFile { path: uri.into() }.into());
@@ -449,19 +519,16 @@ impl ObjectSource for AzureBlobSource {
             io_stats_on_bytestream(Box::pin(stream), io_stats),
             None,
             None,
+            None,
         ))
     }
 
     async fn get_size(&self, uri: &str, io_stats: Option<IOStatsRef>) -> super::Result<usize> {
-        let parsed = url::Url::parse(uri).with_context(|_| InvalidUrlSnafu { path: uri })?;
-        let container = match parsed.host_str() {
-            Some(s) => Ok(s),
-            None => Err(Error::InvalidUrl {
-                path: uri.into(),
-                source: url::ParseError::EmptyHost,
-            }),
-        }?;
-        let key = parsed.path();
+        let (_, container_and_key) = parse_azure_uri(uri)?;
+        let (container, key) = container_and_key.ok_or_else(|| Error::InvalidUrl {
+            path: uri.into(),
+            source: url::ParseError::EmptyHost,
+        })?;
 
         if key.is_empty() {
             return Err(Error::NotAFile { path: uri.into() }.into());
@@ -511,39 +578,23 @@ impl ObjectSource for AzureBlobSource {
         _page_size: Option<i32>,
         io_stats: Option<IOStatsRef>,
     ) -> super::Result<BoxStream<super::Result<FileMetadata>>> {
-        let uri = url::Url::parse(uri).with_context(|_| InvalidUrlSnafu { path: uri })?;
+        let (protocol, container_and_key) = parse_azure_uri(uri)?;
 
-        // path can be root (buckets) or path prefix within a bucket.
-        let container = {
-            // "Container" is Azure's name for Bucket.
-            //
-            // fsspec supports two URI formats; for compatibility, we will support both as well.
-            // PROTOCOL://container/path-part/file
-            // PROTOCOL://container@account.dfs.core.windows.net/path-part/file
-            // See https://github.com/fsspec/adlfs/ for more details
-            let username = uri.username();
-            match username {
-                "" => uri.host_str(),
-                _ => Some(username),
-            }
-        };
-
-        // fsspec supports multiple URI protocol strings for Azure: az:// and abfs://.
-        // NB: It's unclear if there is a semantic difference between the protocols
-        // or if there is a standard for the behaviour either;
-        // here, we will treat them both the same, but persist whichever protocol string was used.
-        let protocol = uri.scheme();
-
-        match container {
+        match container_and_key {
             // List containers.
-            None => Ok(self.list_containers_stream(protocol, io_stats).await),
+            None => Ok(self
+                .list_containers_stream(protocol.as_str(), io_stats)
+                .await),
             // List a path within a container.
-            Some(container_name) => {
-                let prefix = uri.path();
-                Ok(self
-                    .list_directory_stream(protocol, container_name, prefix, posix, io_stats)
-                    .await)
-            }
+            Some((container_name, key)) => Ok(self
+                .list_directory_stream(
+                    protocol.as_str(),
+                    container_name.as_str(),
+                    key.as_str(),
+                    posix,
+                    io_stats,
+                )
+                .await),
         }
     }
 
