@@ -335,7 +335,7 @@ async fn read_json_single_into_stream(
                         .unwrap_or(64),
                 )
             }
-            GetResult::Stream(stream, _, _) => (
+            GetResult::Stream(stream, ..) => (
                 Box::new(StreamReader::new(stream)),
                 // Use user-provided buffer size, falling back to 8 * the user-provided chunk size if that exists, otherwise falling back to 512 KiB as the default.
                 read_options
@@ -528,7 +528,7 @@ mod tests {
             Some(projection) => (
                 projection
                     .iter()
-                    .map(|c| field_map.remove(c.as_str()).unwrap())
+                    .map(|c| field_map.swap_remove(c.as_str()).unwrap())
                     .collect::<Vec<_>>()
                     .into(),
                 true,
