@@ -30,6 +30,7 @@ use daft_io::{IOClient, IOConfig, IOStatsContext, IOStatsRef};
 use daft_stats::TableStatistics;
 use daft_stats::{PartitionSpec, TableMetadata};
 
+#[derive(Debug)]
 pub(crate) enum TableState {
     Unloaded(Arc<ScanTask>),
     Loaded(Arc<Vec<Table>>),
@@ -59,7 +60,9 @@ impl Display for TableState {
         }
     }
 }
-pub(crate) struct MicroPartition {
+
+#[derive(Debug)]
+pub struct MicroPartition {
     /// Schema of the MicroPartition
     ///
     /// This is technically redundant with the schema in `state`:
@@ -172,7 +175,7 @@ fn materialize_scan_task(
                         col_names
                             .as_ref()
                             .map(|cols| cols.iter().map(|col| col.to_string()).collect()),
-                        None,
+                        Some(schema_of_file),
                         scan_task.pushdowns.filters.clone(),
                     );
                     let parse_options = CsvParseOptions::new_with_defaults(
@@ -180,6 +183,7 @@ fn materialize_scan_task(
                         cfg.delimiter,
                         cfg.double_quote,
                         cfg.quote,
+                        cfg.allow_variable_columns,
                         cfg.escape_char,
                         cfg.comment,
                     )
@@ -616,10 +620,18 @@ impl MicroPartition {
         self.schema.names()
     }
 
+    pub fn schema(&self) -> SchemaRef {
+        self.schema.clone()
+    }
+
     pub fn len(&self) -> usize {
         self.metadata.length
     }
 
+    pub fn is_empty(&self) -> bool {
+        self.len() == 0
+    }
+
     pub fn size_bytes(&self) -> DaftResult<Option<usize>> {
         let guard = self.state.lock().unwrap();
         let size_bytes = if let TableState::Loaded(tables) = guard.deref() {
@@ -657,7 +669,7 @@ impl MicroPartition {
         }
     }
 
-    pub(crate) fn concat_or_get(&self, io_stats: IOStatsRef) -> crate::Result<Arc<Vec<Table>>> {
+    pub fn concat_or_get(&self, io_stats: IOStatsRef) -> crate::Result<Arc<Vec<Table>>> {
         let tables = self.tables_or_read(io_stats)?;
         if tables.len() <= 1 {
             return Ok(tables);
@@ -678,7 +690,7 @@ impl MicroPartition {
         }
     }
 
-    pub(crate) fn add_monotonically_increasing_id(
+    pub fn add_monotonically_increasing_id(
         &self,
         partition_num: u64,
         column_name: &str,
