@@ -1,4 +1,7 @@
-use std::{collections::HashSet, sync::Arc};
+use std::{
+    collections::{HashMap, HashSet},
+    sync::Arc,
+};
 
 use crate::{
     logical_ops,
@@ -7,10 +10,9 @@ use crate::{
     partitioning::{
         HashRepartitionConfig, IntoPartitionsConfig, RandomShuffleConfig, RepartitionSpec,
     },
-    physical_planner::plan,
     sink_info::{OutputFileInfo, SinkInfo},
     source_info::SourceInfo,
-    PhysicalPlanScheduler, ResourceRequest,
+    ResourceRequest,
 };
 use common_error::{DaftError, DaftResult};
 use common_io_config::IOConfig;
@@ -23,10 +25,8 @@ use daft_scan::{file_format::FileFormat, Pushdowns, ScanExternalInfo, ScanOperat
 
 #[cfg(feature = "python")]
 use {
-    crate::physical_planner::python::AdaptivePhysicalPlanScheduler,
     crate::sink_info::{CatalogInfo, IcebergCatalogInfo},
-    crate::{physical_plan::PhysicalPlanRef, source_info::InMemoryInfo},
-    common_daft_config::PyDaftExecutionConfig,
+    crate::source_info::InMemoryInfo,
     daft_core::python::schema::PySchema,
     daft_dsl::python::PyExpr,
     daft_scan::python::pylib::ScanOperatorHandle,
@@ -60,6 +60,7 @@ fn check_for_agg(expr: &ExprRef) -> bool {
         BinaryOp { left, right, .. } => check_for_agg(left) || check_for_agg(right),
         Function { inputs, .. } => inputs.iter().any(check_for_agg),
         IsIn(l, r) | FillNull(l, r) => check_for_agg(l) || check_for_agg(r),
+        Between(v, l, u) => check_for_agg(v) || check_for_agg(l) || check_for_agg(u),
         IfElse {
             if_true,
             if_false,
@@ -217,22 +218,32 @@ impl LogicalPlanBuilder {
     ) -> DaftResult<Self> {
         err_if_agg("with_columns", &columns)?;
 
-        let new_col_names = columns.iter().map(|e| e.name()).collect::<HashSet<&str>>();
-
-        let mut exprs = self
-            .schema()
-            .fields
+        let fields = &self.schema().fields;
+        let current_col_names = fields
             .iter()
-            .filter_map(|(name, _)| {
-                if new_col_names.contains(name.as_str()) {
-                    None
-                } else {
-                    Some(col(name.clone()))
-                }
+            .map(|(name, _)| name.as_str())
+            .collect::<HashSet<_>>();
+        let new_col_name_and_exprs = columns
+            .iter()
+            .map(|e| (e.name(), e.clone()))
+            .collect::<HashMap<_, _>>();
+
+        let mut exprs = fields
+            .iter()
+            .map(|(name, _)| {
+                new_col_name_and_exprs
+                    .get(name.as_str())
+                    .cloned()
+                    .unwrap_or_else(|| col(name.clone()))
             })
             .collect::<Vec<_>>();
 
-        exprs.extend(columns);
+        exprs.extend(
+            columns
+                .iter()
+                .filter(|e| !current_col_names.contains(e.name()))
+                .cloned(),
+        );
 
         let logical_plan: LogicalPlan =
             logical_ops::Project::try_new(self.plan.clone(), exprs, resource_request)?.into();
@@ -510,7 +521,7 @@ impl LogicalPlanBuilder {
         path: String,
         columns_name: Vec<String>,
         mode: String,
-        current_version: i32,
+        version: i32,
         large_dtypes: bool,
         io_config: Option<IOConfig>,
     ) -> DaftResult<Self> {
@@ -519,7 +530,7 @@ impl LogicalPlanBuilder {
             catalog: crate::sink_info::CatalogType::DeltaLake(DeltaLakeCatalogInfo {
                 path,
                 mode,
-                current_version,
+                version,
                 large_dtypes,
                 io_config,
             }),
@@ -559,7 +570,7 @@ impl From<LogicalPlan> for LogicalPlanBuilder {
 #[derive(Debug)]
 pub struct PyLogicalPlanBuilder {
     // Internal logical plan builder.
-    builder: LogicalPlanBuilder,
+    pub builder: LogicalPlanBuilder,
 }
 
 impl PyLogicalPlanBuilder {
@@ -804,7 +815,7 @@ impl PyLogicalPlanBuilder {
         path: String,
         columns_name: Vec<String>,
         mode: String,
-        current_version: i32,
+        version: i32,
         large_dtypes: bool,
         io_config: Option<common_io_config::python::IOConfig>,
     ) -> PyResult<Self> {
@@ -814,7 +825,7 @@ impl PyLogicalPlanBuilder {
                 path,
                 columns_name,
                 mode,
-                current_version,
+                version,
                 large_dtypes,
                 io_config.map(|cfg| cfg.config),
             )?
@@ -856,38 +867,9 @@ impl PyLogicalPlanBuilder {
         })
     }
 
-    /// Finalize the logical plan, translate the logical plan to a physical plan, and return
-    /// a physical plan scheduler that's capable of launching the work necessary to compute the output
-    /// of the physical plan.
-    pub fn to_physical_plan_scheduler(
-        &self,
-        py: Python,
-        cfg: PyDaftExecutionConfig,
-    ) -> PyResult<PhysicalPlanScheduler> {
-        py.allow_threads(|| {
-            let logical_plan = self.builder.build();
-            let physical_plan: PhysicalPlanRef = plan(logical_plan, cfg.config.clone())?;
-            Ok(physical_plan.into())
-        })
-    }
-
     pub fn repr_ascii(&self, simple: bool) -> PyResult<String> {
         Ok(self.builder.repr_ascii(simple))
     }
-
-    pub fn to_adaptive_physical_plan_scheduler(
-        &self,
-        py: Python,
-        cfg: PyDaftExecutionConfig,
-    ) -> PyResult<AdaptivePhysicalPlanScheduler> {
-        py.allow_threads(|| {
-            let logical_plan = self.builder.build();
-            Ok(AdaptivePhysicalPlanScheduler::new(
-                logical_plan,
-                cfg.config.clone(),
-            ))
-        })
-    }
 }
 
 impl From<LogicalPlanBuilder> for PyLogicalPlanBuilder {
