@@ -5,12 +5,15 @@ mod explode;
 mod filter;
 mod join;
 mod limit;
+mod monotonically_increasing_id;
+mod pivot;
 mod project;
 mod repartition;
 mod sample;
 mod sink;
 mod sort;
 mod source;
+mod unpivot;
 
 pub use agg::Aggregate;
 pub use concat::Concat;
@@ -19,9 +22,12 @@ pub use explode::Explode;
 pub use filter::Filter;
 pub use join::Join;
 pub use limit::Limit;
+pub use monotonically_increasing_id::MonotonicallyIncreasingId;
+pub use pivot::Pivot;
 pub use project::Project;
 pub use repartition::Repartition;
 pub use sample::Sample;
 pub use sink::Sink;
 pub use sort::Sort;
 pub use source::Source;
+pub use unpivot::Unpivot;
