@@ -3,6 +3,8 @@ mod broadcast_join;
 mod coalesce;
 mod concat;
 mod csv;
+#[cfg(feature = "python")]
+mod deltalake_write;
 mod empty_scan;
 mod explode;
 mod fanout;
@@ -10,22 +12,29 @@ mod filter;
 mod flatten;
 mod hash_join;
 #[cfg(feature = "python")]
+mod iceberg_write;
 mod in_memory;
 mod json;
 mod limit;
+mod monotonically_increasing_id;
 mod parquet;
+mod pivot;
 mod project;
 mod reduce;
 mod sample;
 mod scan;
 mod sort;
+mod sort_merge_join;
 mod split;
+mod unpivot;
 
 pub use agg::Aggregate;
 pub use broadcast_join::BroadcastJoin;
 pub use coalesce::Coalesce;
 pub use concat::Concat;
-pub use csv::{TabularScanCsv, TabularWriteCsv};
+pub use csv::TabularWriteCsv;
+#[cfg(feature = "python")]
+pub use deltalake_write::DeltaLakeWrite;
 pub use empty_scan::EmptyScan;
 pub use explode::Explode;
 pub use fanout::{FanoutByHash, FanoutByRange, FanoutRandom};
@@ -33,13 +42,18 @@ pub use filter::Filter;
 pub use flatten::Flatten;
 pub use hash_join::HashJoin;
 #[cfg(feature = "python")]
+pub use iceberg_write::IcebergWrite;
 pub use in_memory::InMemoryScan;
-pub use json::{TabularScanJson, TabularWriteJson};
+pub use json::TabularWriteJson;
 pub use limit::Limit;
-pub use parquet::{TabularScanParquet, TabularWriteParquet};
+pub use monotonically_increasing_id::MonotonicallyIncreasingId;
+pub use parquet::TabularWriteParquet;
+pub use pivot::Pivot;
 pub use project::Project;
 pub use reduce::ReduceMerge;
 pub use sample::Sample;
 pub use scan::TabularScan;
 pub use sort::Sort;
+pub use sort_merge_join::SortMergeJoin;
 pub use split::Split;
+pub use unpivot::Unpivot;
