@@ -8,7 +8,7 @@ use std::hash::Hash;
 use std::{collections::BTreeMap, str::FromStr, sync::Arc};
 #[cfg(feature = "python")]
 use {
-    crate::py_object_serde::{deserialize_py_object, serialize_py_object},
+    common_py_serde::{deserialize_py_object, serialize_py_object},
     daft_core::python::{datatype::PyTimeUnit, field::PyField},
     pyo3::{
         pyclass, pyclass::CompareOp, pymethods, types::PyBytes, IntoPy, PyObject, PyResult,
@@ -183,6 +183,7 @@ pub struct CsvSourceConfig {
     pub quote: Option<char>,
     pub escape_char: Option<char>,
     pub comment: Option<char>,
+    pub allow_variable_columns: bool,
     pub buffer_size: Option<usize>,
     pub chunk_size: Option<usize>,
 }
@@ -204,6 +205,10 @@ impl CsvSourceConfig {
         if let Some(comment) = self.comment {
             res.push(format!("Comment = {}", comment));
         }
+        res.push(format!(
+            "Allow_variable_columns = {}",
+            self.allow_variable_columns
+        ));
         if let Some(buffer_size) = self.buffer_size {
             res.push(format!("Buffer size = {}", buffer_size));
         }
@@ -230,6 +235,7 @@ impl CsvSourceConfig {
     fn new(
         has_headers: bool,
         double_quote: bool,
+        allow_variable_columns: bool,
         delimiter: Option<char>,
         quote: Option<char>,
         escape_char: Option<char>,
@@ -244,6 +250,7 @@ impl CsvSourceConfig {
             quote,
             escape_char,
             comment,
+            allow_variable_columns,
             buffer_size,
             chunk_size,
         })
