@@ -4,7 +4,9 @@ mod groups;
 mod hash;
 mod joins;
 mod partition;
+mod pivot;
 mod search_sorted;
 mod sort;
+mod unpivot;
 
 pub use joins::infer_join_schema;
