@@ -9,14 +9,25 @@ import pytest
 import daft
 from daft.table import MicroPartition
 
+# import all conftest
+from tests.integration.io.conftest import *
+
+
+def pytest_addoption(parser):
+    parser.addoption(
+        "--credentials",
+        action="store_true",
+        help="Whether or not the current environment has access to remote storage credentials",
+    )
+
 
 @pytest.fixture(scope="session", autouse=True)
 def set_execution_configs():
     """Sets global Daft config for testing"""
     daft.set_execution_config(
         # Disables merging of ScanTasks
-        merge_scan_tasks_min_size_bytes=0,
-        merge_scan_tasks_max_size_bytes=0,
+        scan_tasks_min_size_bytes=0,
+        scan_tasks_max_size_bytes=0,
     )
 
 
@@ -60,7 +71,29 @@ def uuid_ext_type() -> UuidType:
         "parquet",
     ],
 )
-def make_df(request, tmp_path) -> daft.Dataframe:
+def data_source(request):
+    return request.param
+
+
+@pytest.fixture(scope="function")
+def join_strategy(request):
+    # Modifies the join strategy parametrization to toggle a specialized presorting path for sort-merge joins, where
+    # each side of the join is sorted such that their boundaries will align.
+    if request.param != "sort_merge_aligned_boundaries":
+        yield request.param
+    else:
+        old_execution_config = daft.context.get_context().daft_execution_config
+        try:
+            daft.set_execution_config(
+                sort_merge_join_sort_with_aligned_boundaries=True,
+            )
+            yield "sort_merge"
+        finally:
+            daft.set_execution_config(old_execution_config)
+
+
+@pytest.fixture(scope="function")
+def make_df(data_source, tmp_path) -> daft.Dataframe:
     """Makes a dataframe when provided with data"""
 
     def _make_df(
@@ -79,7 +112,7 @@ def make_df(request, tmp_path) -> daft.Dataframe:
         else:
             raise NotImplementedError(f"make_df not implemented for input type: {type(data)}")
 
-        variant = request.param
+        variant = data_source
         if variant == "arrow":
             df = daft.from_arrow(pa_table)
             if repartition != 1:
