@@ -8,6 +8,7 @@ import pytest
 
 import daft
 from daft import col
+from daft.context import get_context
 from daft.datatype import DataType
 from daft.errors import ExpressionTypeError
 from daft.utils import freeze
@@ -25,12 +26,12 @@ def test_agg_global(make_df, repartition_nparts):
     )
     daft_df = daft_df.agg(
         [
-            (col("values").alias("sum"), "sum"),
-            (col("values").alias("mean"), "mean"),
-            (col("values").alias("min"), "min"),
-            (col("values").alias("max"), "max"),
-            (col("values").alias("count"), "count"),
-            (col("values").alias("list"), "list"),
+            col("values").sum().alias("sum"),
+            col("values").mean().alias("mean"),
+            col("values").min().alias("min"),
+            col("values").max().alias("max"),
+            col("values").count().alias("count"),
+            col("values").agg_list().alias("list"),
         ]
     )
     expected = {"sum": [3], "mean": [1.5], "min": [1], "max": [2], "count": [2], "list": [[1, None, 2]]}
@@ -57,12 +58,12 @@ def test_agg_global_all_null(make_df, repartition_nparts):
     )
     daft_df = daft_df.where(col("id") != 0).agg(
         [
-            (col("values").alias("sum"), "sum"),
-            (col("values").alias("mean"), "mean"),
-            (col("values").alias("min"), "min"),
-            (col("values").alias("max"), "max"),
-            (col("values").alias("count"), "count"),
-            (col("values").alias("list"), "list"),
+            col("values").sum().alias("sum"),
+            col("values").mean().alias("mean"),
+            col("values").min().alias("min"),
+            col("values").max().alias("max"),
+            col("values").count().alias("count"),
+            col("values").agg_list().alias("list"),
         ]
     )
     expected = {
@@ -90,12 +91,12 @@ def test_agg_global_empty(make_df):
     )
     daft_df = daft_df.where(col("id") != 0).agg(
         [
-            (col("values").alias("sum"), "sum"),
-            (col("values").alias("mean"), "mean"),
-            (col("values").alias("min"), "min"),
-            (col("values").alias("max"), "max"),
-            (col("values").alias("count"), "count"),
-            (col("values").alias("list"), "list"),
+            col("values").sum().alias("sum"),
+            col("values").mean().alias("mean"),
+            col("values").min().alias("min"),
+            col("values").max().alias("max"),
+            col("values").count().alias("count"),
+            col("values").agg_list().alias("list"),
         ]
     )
     expected = {
@@ -124,12 +125,12 @@ def test_agg_groupby(make_df, repartition_nparts):
     )
     daft_df = daft_df.groupby("group").agg(
         [
-            (col("values").alias("sum"), "sum"),
-            (col("values").alias("mean"), "mean"),
-            (col("values").alias("min"), "min"),
-            (col("values").alias("max"), "max"),
-            (col("values").alias("count"), "count"),
-            (col("values").alias("list"), "list"),
+            col("values").sum().alias("sum"),
+            col("values").mean().alias("mean"),
+            col("values").min().alias("min"),
+            col("values").max().alias("max"),
+            col("values").count().alias("count"),
+            col("values").agg_list().alias("list"),
         ]
     )
     expected = {
@@ -169,12 +170,12 @@ def test_agg_groupby_all_null(make_df, repartition_nparts):
     daft_df = daft_df.where(col("id") != 0)
     daft_df = daft_df.groupby(col("group")).agg(
         [
-            (col("values").alias("sum"), "sum"),
-            (col("values").alias("mean"), "mean"),
-            (col("values").alias("min"), "min"),
-            (col("values").alias("max"), "max"),
-            (col("values").alias("count"), "count"),
-            (col("values").alias("list"), "list"),
+            col("values").sum().alias("sum"),
+            col("values").mean().alias("mean"),
+            col("values").min().alias("min"),
+            col("values").max().alias("max"),
+            col("values").count().alias("count"),
+            col("values").agg_list().alias("list"),
         ]
     )
 
@@ -207,11 +208,7 @@ def test_agg_groupby_null_type_column(make_df):
     daft_df = daft_df.groupby(col("group"))
 
     with pytest.raises(ValueError):
-        daft_df.agg(
-            [
-                (col("values").alias("sum"), "sum"),
-            ]
-        )
+        daft_df.agg(col("values").sum().alias("sum"))
 
 
 @pytest.mark.parametrize("repartition_nparts", [1, 2, 5])
@@ -225,11 +222,7 @@ def test_null_groupby_keys(make_df, repartition_nparts):
         repartition=repartition_nparts,
     )
 
-    daft_df = daft_df.groupby(col("group")).agg(
-        [
-            (col("values").alias("mean"), "mean"),
-        ]
-    )
+    daft_df = daft_df.groupby(col("group")).agg(col("values").mean().alias("mean"))
 
     expected = {
         "group": [0, 1, 2, None],
@@ -256,10 +249,8 @@ def test_all_null_groupby_keys(make_df, repartition_nparts):
         daft_df.with_column("group", daft_df["group"].cast(DataType.int64()))
         .groupby(col("group"))
         .agg(
-            [
-                (col("values").alias("mean"), "mean"),
-                (col("values").alias("list"), "list"),
-            ]
+            col("values").agg_list().alias("list"),
+            col("values").mean().alias("mean"),
         )
     )
 
@@ -297,12 +288,12 @@ def test_agg_groupby_empty(make_df):
     daft_df = daft_df.where(col("id") != 0)
     daft_df = daft_df.groupby(col("group")).agg(
         [
-            (col("values").alias("sum"), "sum"),
-            (col("values").alias("mean"), "mean"),
-            (col("values").alias("min"), "min"),
-            (col("values").alias("max"), "max"),
-            (col("values").alias("count"), "count"),
-            (col("values").alias("list"), "list"),
+            col("values").sum().alias("sum"),
+            col("values").mean().alias("mean"),
+            col("values").min().alias("min"),
+            col("values").max().alias("max"),
+            col("values").count().alias("count"),
+            col("values").agg_list().alias("list"),
         ]
     )
 
@@ -334,8 +325,8 @@ def test_agg_pyobjects():
     df = df.into_partitions(2)
     df = df.agg(
         [
-            (col("objs").alias("count"), "count"),
-            (col("objs").alias("list"), "list"),
+            col("objs").count().alias("count"),
+            col("objs").agg_list().alias("list"),
         ]
     )
     df.collect()
@@ -353,8 +344,8 @@ def test_groupby_agg_pyobjects():
         df.groupby(col("groups"))
         .agg(
             [
-                (col("objects").alias("count"), "count"),
-                (col("objects").alias("list"), "list"),
+                col("objects").count().alias("count"),
+                col("objects").agg_list().alias("list"),
             ]
         )
         .sort(col("groups"))
@@ -365,3 +356,46 @@ def test_groupby_agg_pyobjects():
     assert res["groups"] == [1, 2]
     assert res["count"] == [2, 1]
     assert res["list"] == [[objects[0], objects[2], objects[4]], [objects[1], objects[3]]]
+
+
+@pytest.mark.parametrize("shuffle_aggregation_default_partitions", [None, 20])
+def test_groupby_result_partitions_smaller_than_input(shuffle_aggregation_default_partitions):
+    if shuffle_aggregation_default_partitions is None:
+        min_partitions = get_context().daft_execution_config.shuffle_aggregation_default_partitions
+    else:
+        daft.set_execution_config(shuffle_aggregation_default_partitions=shuffle_aggregation_default_partitions)
+        min_partitions = shuffle_aggregation_default_partitions
+
+    for partition_size in [1, min_partitions, min_partitions + 1]:
+        df = daft.from_pydict(
+            {"group": [i for i in range(min_partitions + 1)], "value": [i for i in range(min_partitions + 1)]}
+        )
+        df = df.into_partitions(partition_size)
+
+        df = df.groupby(col("group")).agg(
+            [
+                col("value").sum().alias("sum"),
+                col("value").mean().alias("mean"),
+                col("value").min().alias("min"),
+            ]
+        )
+
+        df = df.collect()
+
+        assert df.num_partitions() == min(min_partitions, partition_size)
+
+
+def test_agg_deprecation():
+    with pytest.deprecated_call():
+        df = daft.from_pydict({"a": [1, 2, 3], "b": [True, False, True]})
+        df = df.agg([("a", "sum"), ("b", "count")])
+        df.collect()
+
+        assert df.to_pydict() == {"a": [6], "b": [3]}
+
+    with pytest.deprecated_call():
+        df = daft.from_pydict({"a": [1, 2, 3], "b": [True, False, True]})
+        df = df.groupby("b").agg([("a", "sum")])
+        df.collect()
+
+        assert df.to_pydict() == {"b": [True, False], "a": [4, 2]}
