@@ -2,18 +2,11 @@ from __future__ import annotations
 
 import datetime
 import itertools
-import sys
-
-import pytz
-
-if sys.version_info < (3, 8):
-    pass
-else:
-    pass
 from typing import Callable
 
 import pyarrow as pa
 import pytest
+import pytz
 
 from daft.datatype import DataType
 from daft.expressions import Expression, ExpressionsProjection
@@ -215,7 +208,6 @@ def has_supertype(dt1: DataType, dt2: DataType) -> bool:
         return True
 
     for x, y in ((dt1, dt2), (dt2, dt1)):
-
         # --- Common types across hierarchies ---
         either_null = x == DataType.null()
         either_string_and_other_not_binary = x == DataType.string() and y != DataType.binary()
@@ -226,6 +218,7 @@ def has_supertype(dt1: DataType, dt2: DataType) -> bool:
 
         # --- Across type hierarchies ---
         date_and_numeric = x == DataType.date() and is_numeric(y)
+        time_and_numeric = x == (DataType.time("us") or DataType.time("ns")) and is_numeric(y)
         timestamp_and_big_numeric = x._is_temporal_type() and is_numeric_bitwidth_gte_32(y)
 
         if (
@@ -234,6 +227,7 @@ def has_supertype(dt1: DataType, dt2: DataType) -> bool:
             or both_numeric
             or both_temporal
             or date_and_numeric
+            or time_and_numeric
             or timestamp_and_big_numeric
         ):
             return True
