@@ -88,6 +88,25 @@ def test_series_date_size_bytes(size, with_nulls) -> None:
     assert s.size_bytes() == get_total_buffer_size(data)
 
 
+@pytest.mark.parametrize("size", [0, 1, 2, 8, 9, 16])
+@pytest.mark.parametrize("with_nulls", [True, False])
+@pytest.mark.parametrize("precision", ["us", "ns"])
+def test_series_time_size_bytes(size, with_nulls, precision) -> None:
+    from datetime import time
+
+    pydata = [time(i, i, i, i) for i in range(size)]
+
+    if with_nulls and size > 0:
+        data = pa.array(pydata[:-1] + [None], pa.time64(precision))
+    else:
+        data = pa.array(pydata, pa.time64(precision))
+
+    s = Series.from_arrow(data)
+
+    assert s.datatype() == DataType.time(precision)
+    assert s.size_bytes() == get_total_buffer_size(data)
+
+
 @pytest.mark.parametrize("size", [0, 1, 2, 8, 9, 16])
 @pytest.mark.parametrize("with_nulls", [True, False])
 def test_series_binary_size_bytes(size, with_nulls) -> None:
@@ -157,7 +176,7 @@ def test_series_struct_size_bytes(size, with_nulls) -> None:
         for i in range(size)
     ]
 
-    # Artifically inject nulls if required as a top-level StructArray Null entry
+    # Artificially inject nulls if required as a top-level StructArray Null entry
     if with_nulls and size > 0:
         data = pa.array(pydata[:-1] + [None], type=dtype)
     else:
