@@ -39,11 +39,11 @@ test_table_count_cases = [
     ],
 )
 def test_multipartition_count_empty(mp):
-    counted = mp.agg([col("a")._count()])
+    counted = mp.agg([col("a").count()])
     assert len(counted) == 1
     assert counted.to_pydict() == {"a": [0]}
 
-    counted = mp.agg([col("a")._count()], group_by=[col("b")])
+    counted = mp.agg([col("a").count()], group_by=[col("b")])
     assert len(counted) == 0
     assert counted.to_pydict() == {"b": [], "a": []}
 
@@ -62,11 +62,11 @@ def test_multipartition_count_empty(mp):
     ],
 )
 def test_multipartition_count(mp):
-    counted = mp.agg([col("a")._count()])
+    counted = mp.agg([col("a").count()])
     assert len(counted) == 1
     assert counted.to_pydict() == {"a": [2]}
 
-    counted = mp.agg([col("a")._count()], group_by=[col("b")])
+    counted = mp.agg([col("a").count()], group_by=[col("b")])
     assert len(counted) == 2
     assert counted.to_pydict() == {"b": ["a", "b"], "a": [1, 1]}
 
@@ -79,7 +79,7 @@ def test_table_count(idx_dtype, case) -> None:
         input = [datetime.date(2020 + x, 1 + x, 1 + x) if x is not None else None for x in input]
     daft_table = MicroPartition.from_pydict({"input": input})
     daft_table = daft_table.eval_expression_list([col("input").cast(idx_dtype)])
-    daft_table = daft_table.eval_expression_list([col("input").alias("count")._count()])
+    daft_table = daft_table.eval_expression_list([col("input").alias("count").count()])
 
     res = daft_table.to_pydict()
     assert res == expected
@@ -89,7 +89,7 @@ def test_table_count(idx_dtype, case) -> None:
 def test_table_count_nulltype(length) -> None:
     daft_table = MicroPartition.from_pydict({"input": [None] * length})
     daft_table = daft_table.eval_expression_list([col("input").cast(DataType.null())])
-    daft_table = daft_table.eval_expression_list([col("input").alias("count")._count()])
+    daft_table = daft_table.eval_expression_list([col("input").alias("count").count()])
 
     res = daft_table.to_pydict()["count"]
     assert res == [0]
@@ -97,7 +97,7 @@ def test_table_count_nulltype(length) -> None:
 
 def test_table_count_pyobject() -> None:
     daft_table = MicroPartition.from_pydict({"objs": [object(), object(), None, object(), None]})
-    daft_table = daft_table.eval_expression_list([col("objs").alias("count")._count()])
+    daft_table = daft_table.eval_expression_list([col("objs").alias("count").count()])
 
     res = daft_table.to_pydict()["count"]
     assert res == [3]
@@ -123,8 +123,8 @@ def test_table_minmax_numerics(idx_dtype, case) -> None:
     daft_table = daft_table.eval_expression_list([col("input").cast(idx_dtype)])
     daft_table = daft_table.eval_expression_list(
         [
-            col("input").alias("min")._min(),
-            col("input").alias("max")._max(),
+            col("input").alias("min").min(),
+            col("input").alias("max").max(),
         ]
     )
 
@@ -150,8 +150,8 @@ def test_table_minmax_string(idx_dtype, case) -> None:
     daft_table = daft_table.eval_expression_list([col("input").cast(idx_dtype)])
     daft_table = daft_table.eval_expression_list(
         [
-            col("input").alias("min")._min(),
-            col("input").alias("max")._max(),
+            col("input").alias("min").min(),
+            col("input").alias("max").max(),
         ]
     )
 
@@ -175,8 +175,8 @@ def test_table_minmax_bool(case) -> None:
     daft_table = daft_table.eval_expression_list([col("input").cast(DataType.bool())])
     daft_table = daft_table.eval_expression_list(
         [
-            col("input").alias("min")._min(),
-            col("input").alias("max")._max(),
+            col("input").alias("min").min(),
+            col("input").alias("max").max(),
         ]
     )
 
@@ -202,8 +202,8 @@ def test_table_sum_mean(idx_dtype, case) -> None:
     daft_table = daft_table.eval_expression_list([col("input").cast(idx_dtype)])
     daft_table = daft_table.eval_expression_list(
         [
-            col("input").alias("sum")._sum(),
-            col("input").alias("mean")._mean(),
+            col("input").alias("sum").sum(),
+            col("input").alias("mean").mean(),
         ]
     )
 
@@ -220,7 +220,7 @@ def test_table_sum_upcast(nptype) -> None:
             "mins": np.full(128, fill_value=np.iinfo(nptype).min, dtype=nptype),
         }
     )
-    daft_table = daft_table.eval_expression_list([col("maxes")._sum(), col("mins")._sum()])
+    daft_table = daft_table.eval_expression_list([col("maxes").sum(), col("mins").sum()])
     pydict = daft_table.to_pydict()
     assert pydict["maxes"] == [128 * np.iinfo(nptype).max]
     assert pydict["mins"] == [128 * np.iinfo(nptype).min]
@@ -229,7 +229,73 @@ def test_table_sum_upcast(nptype) -> None:
 def test_table_sum_badtype() -> None:
     daft_table = MicroPartition.from_pydict({"a": ["str1", "str2"]})
     with pytest.raises(ValueError):
-        daft_table = daft_table.eval_expression_list([col("a")._sum()])
+        daft_table = daft_table.eval_expression_list([col("a").sum()])
+
+
+test_micropartition_any_value_cases = [
+    (
+        MicroPartition.from_pydict({"a": [None, 1, None, None], "b": ["a", "a", "b", "b"]}),  # 1 table
+        {"a": True, "b": True},
+        {"a": False, "b": True},
+    ),
+    (
+        MicroPartition.concat(
+            [
+                MicroPartition.from_pydict({"a": np.array([]).astype(np.int64), "b": pa.array([], type=pa.string())}),
+                MicroPartition.from_pydict({"a": [None, 3, None], "b": ["a", "b", "b"]}),
+            ]
+        ),  # 2 tables
+        {"a": True, "b": True},
+        {"a": True, "b": False},
+    ),
+    (
+        MicroPartition.concat(
+            [
+                MicroPartition.from_pydict({"a": np.array([]).astype(np.int64), "b": pa.array([], type=pa.string())}),
+                MicroPartition.from_pydict({"a": [None, 3, None], "b": ["a", "b", "b"]}),
+                MicroPartition.from_pydict({"a": [1], "b": ["a"]}),
+            ]
+        ),  # 3 tables
+        {"a": True, "b": True},
+        {"a": False, "b": False},
+    ),
+]
+
+
+@pytest.mark.parametrize("mp,expected_nulls,expected_no_nulls", test_micropartition_any_value_cases)
+def test_micropartition_any_value(mp, expected_nulls, expected_no_nulls):
+    any_values = mp.agg([col("a").any_value(False)], group_by=[col("b")]).to_pydict()
+    assert len(any_values["b"]) == len(expected_nulls)
+    for k, v in zip(any_values["b"], any_values["a"]):
+        assert expected_nulls[k] or v is not None
+
+    any_values = mp.agg([col("a").any_value(True)], group_by=[col("b")]).to_pydict()
+    assert len(any_values["b"]) == len(expected_no_nulls)
+    for k, v in zip(any_values["b"], any_values["a"]):
+        assert expected_no_nulls[k] or v is not None
+
+
+test_table_any_value_cases = [
+    ({"a": [1], "b": ["a"]}, {"a": False}, {"a": False}),
+    ({"a": [None], "b": ["a"]}, {"a": True}, {"a": True}),
+    ({"a": [None, 1], "b": ["a", "a"]}, {"a": True}, {"a": False}),
+    ({"a": [1, None, 2], "b": ["a", "b", "b"]}, {"a": False, "b": True}, {"a": False, "b": False}),
+]
+
+
+@pytest.mark.parametrize("case,expected_nulls,expected_no_nulls", test_table_any_value_cases)
+def test_table_any_value(case, expected_nulls, expected_no_nulls):
+    daft_table = MicroPartition.from_pydict(case)
+
+    any_values = daft_table.agg([col("a").any_value(False)], group_by=[col("b")]).to_pydict()
+    assert len(any_values["b"]) == len(expected_nulls)
+    for k, v in zip(any_values["b"], any_values["a"]):
+        assert expected_nulls[k] or v is not None
+
+    any_values = daft_table.agg([col("a").any_value(True)], group_by=[col("b")]).to_pydict()
+    assert len(any_values["b"]) == len(expected_no_nulls)
+    for k, v in zip(any_values["b"], any_values["a"]):
+        assert expected_no_nulls[k] or v is not None
 
 
 test_table_agg_global_cases = [
@@ -266,12 +332,12 @@ def test_table_agg_global(case) -> None:
     daft_table = MicroPartition.from_pydict({"input": input})
     daft_table = daft_table.agg(
         [
-            col("input").cast(DataType.int32()).alias("count")._count(),
-            col("input").cast(DataType.int32()).alias("sum")._sum(),
-            col("input").cast(DataType.int32()).alias("mean")._mean(),
-            col("input").cast(DataType.int32()).alias("min")._min(),
-            col("input").cast(DataType.int32()).alias("max")._max(),
-            col("input").cast(DataType.int32()).alias("list")._agg_list(),
+            col("input").cast(DataType.int32()).alias("count").count(),
+            col("input").cast(DataType.int32()).alias("sum").sum(),
+            col("input").cast(DataType.int32()).alias("mean").mean(),
+            col("input").cast(DataType.int32()).alias("min").min(),
+            col("input").cast(DataType.int32()).alias("max").max(),
+            col("input").cast(DataType.int32()).alias("list").agg_list(),
         ]
     )
 
@@ -291,7 +357,7 @@ def test_table_agg_groupby_empty(groups_and_aggs) -> None:
     groups, aggs = groups_and_aggs
     daft_table = MicroPartition.from_pydict({"col_A": [], "col_B": []})
     daft_table = daft_table.agg(
-        [col(a)._count() for a in aggs],
+        [col(a).count() for a in aggs],
         [col(g).cast(DataType.int32()) for g in groups],
     )
     res = daft_table.to_pydict()
@@ -304,9 +370,9 @@ test_table_agg_groupby_cases = [
         # Group by strings.
         "groups": ["name"],
         "aggs": [
-            col("cookies").alias("sum")._sum(),
-            col("name").alias("count")._count(),
-            col("cookies").alias("list")._agg_list(),
+            col("cookies").alias("sum").sum(),
+            col("name").alias("count").count(),
+            col("cookies").alias("list").agg_list(),
         ],
         "expected": {
             "name": ["Alice", "Bob", None],
@@ -318,13 +384,13 @@ test_table_agg_groupby_cases = [
     {
         # Group by numbers.
         "groups": ["cookies"],
-        "aggs": [col("name").alias("count")._count()],
+        "aggs": [col("name").alias("count").count()],
         "expected": {"cookies": [2, 5, None], "count": [0, 2, 6]},
     },
     {
         # Group by multicol.
         "groups": ["name", "cookies"],
-        "aggs": [col("name").alias("count")._count()],
+        "aggs": [col("name").alias("count").count()],
         "expected": {
             "name": ["Alice", "Bob", "Bob", None, None, None],
             "cookies": [None, 5, None, 2, 5, None],
@@ -375,7 +441,7 @@ def test_groupby_all_nulls(dtype) -> None:
             "cookies": [1, 2, 3],
         }
     )
-    result_table = daft_table.agg([col("cookies")._sum()], group_by=[col("group")])
+    result_table = daft_table.agg([col("cookies").sum()], group_by=[col("group")])
     assert result_table.to_pydict() == {"group": [None], "cookies": [6]}
 
 
@@ -391,7 +457,7 @@ def test_groupby_numeric_string_bool_some_nulls(dtype) -> None:
             "cookies": [2, 2, 3],
         }
     )
-    result_table = daft_table.agg([col("cookies")._sum()], group_by=[col("group")])
+    result_table = daft_table.agg([col("cookies").sum()], group_by=[col("group")])
     expected_table = MicroPartition.from_pydict(
         {
             "group": Series.from_pylist([1, None]).cast(dtype),
@@ -416,7 +482,7 @@ def test_groupby_numeric_string_bool_no_nulls(dtype) -> None:
             "cookies": [1, 2, 2, 3],
         }
     )
-    result_table = daft_table.agg([col("cookies")._sum()], group_by=[col("group")])
+    result_table = daft_table.agg([col("cookies").sum()], group_by=[col("group")])
     expected_table = MicroPartition.from_pydict(
         {
             "group": Series.from_pylist([0, 1]).cast(dtype),
@@ -436,7 +502,7 @@ def test_groupby_binary_bool_some_nulls() -> None:
             "cookies": [2, 2, 3],
         }
     )
-    result_table = daft_table.agg([col("cookies")._sum()], group_by=[col("group")])
+    result_table = daft_table.agg([col("cookies").sum()], group_by=[col("group")])
     expected_table = MicroPartition.from_pydict(
         {
             "group": Series.from_pylist([b"1", None]),
@@ -456,7 +522,7 @@ def test_groupby_binary_no_nulls() -> None:
             "cookies": [1, 2, 2, 3],
         }
     )
-    result_table = daft_table.agg([col("cookies")._sum()], group_by=[col("group")])
+    result_table = daft_table.agg([col("cookies").sum()], group_by=[col("group")])
     expected_table = MicroPartition.from_pydict(
         {
             "group": Series.from_pylist([b"0", b"1"]),
@@ -476,7 +542,7 @@ def test_groupby_binary_no_nulls_max() -> None:
             "cookies": [b"1", b"2", b"2", b"3"],
         }
     )
-    result_table = daft_table.agg([col("cookies")._max()], group_by=[col("group")])
+    result_table = daft_table.agg([col("cookies").max()], group_by=[col("group")])
     expected_table = MicroPartition.from_pydict(
         {
             "group": Series.from_pylist([b"0", b"1"]),
@@ -506,14 +572,14 @@ def test_groupby_floats_nan(dtype) -> None:
             "cookies": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
         }
     )
-    result_table = daft_table.agg([col("cookies")._count()], group_by=[col("group")])
+    result_table = daft_table.agg([col("cookies").count()], group_by=[col("group")])
     expected_table = MicroPartition.from_pydict(
         {
             "group": Series.from_pylist([None, 1.0, NAN, -INF, INF]).cast(dtype),
             "cookies": [2, 2, 4, 1, 2],
         }
     )
-    # have to sort and compare since `utils.pydict_to_rows` doesnt work on NaNs
+    # have to sort and compare since `utils.pydict_to_rows` doesn't work on NaNs
     for result_col, expected_col in zip(
         result_table.sort([col("group")]).to_pydict(), expected_table.sort([col("group")]).to_pydict()
     ):
@@ -521,6 +587,44 @@ def test_groupby_floats_nan(dtype) -> None:
             assert (r == e) or (math.isnan(r) and math.isnan(e))
 
 
+def test_groupby_timestamp() -> None:
+    daft_table = MicroPartition.from_pydict(
+        {
+            "group": Series.from_pylist(
+                [
+                    datetime.datetime(2020, 1, 1, 0, 0, 0),
+                    datetime.datetime(2020, 1, 1, 0, 30, 0),
+                    datetime.datetime(2020, 1, 1, 0, 59, 59),
+                    datetime.datetime(2020, 1, 1, 1, 0, 0),
+                    datetime.datetime(2020, 1, 1, 1, 30, 0),
+                    datetime.datetime(2020, 1, 1, 1, 59, 59),
+                    datetime.datetime(2020, 1, 1, 2, 0, 0),
+                    datetime.datetime(2020, 1, 1, 2, 30, 0),
+                    datetime.datetime(2020, 1, 1, 2, 59, 59),
+                ]
+            ),
+            "value": [1, 1, 1, 2, 2, 2, 3, 3, 3],
+        }
+    )
+    result_table = daft_table.agg([col("value").sum()], group_by=[col("group").dt.truncate("1 hour")])
+    expected_table = MicroPartition.from_pydict(
+        {
+            "group": Series.from_pylist(
+                [
+                    datetime.datetime(2020, 1, 1, 0, 0, 0),
+                    datetime.datetime(2020, 1, 1, 1, 0, 0),
+                    datetime.datetime(2020, 1, 1, 2, 0, 0),
+                ]
+            ),
+            "value": [3, 6, 9],
+        }
+    )
+
+    assert set(utils.freeze(utils.pydict_to_rows(result_table.to_pydict()))) == set(
+        utils.freeze(utils.pydict_to_rows(expected_table.to_pydict()))
+    )
+
+
 @pytest.mark.parametrize(
     "dtype", daft_nonnull_types + daft_null_types, ids=[f"{_}" for _ in daft_nonnull_types + daft_null_types]
 )
@@ -530,7 +634,7 @@ def test_global_list_aggs(dtype) -> None:
         input = [datetime.date(2020 + x, 1 + x, 1 + x) if x is not None else None for x in input]
     daft_table = MicroPartition.from_pydict({"input": input})
     daft_table = daft_table.eval_expression_list([col("input").cast(dtype)])
-    result = daft_table.eval_expression_list([col("input").alias("list")._agg_list()])
+    result = daft_table.eval_expression_list([col("input").alias("list").agg_list()])
     assert result.get_column("list").datatype() == DataType.list(dtype)
     assert result.to_pydict() == {"list": [daft_table.to_pydict()["input"]]}
 
@@ -538,7 +642,7 @@ def test_global_list_aggs(dtype) -> None:
 def test_global_pyobj_list_aggs() -> None:
     input = [object(), object(), object()]
     table = MicroPartition.from_pydict({"input": input})
-    result = table.eval_expression_list([col("input").alias("list")._agg_list()])
+    result = table.eval_expression_list([col("input").alias("list").agg_list()])
     assert result.get_column("list").datatype() == DataType.python()
     assert result.to_pydict()["list"][0] == input
 
@@ -555,7 +659,7 @@ def test_grouped_list_aggs(dtype) -> None:
         input = [datetime.date(2020 + x, 1 + x, 1 + x) if x is not None else None for x in input]
     daft_table = MicroPartition.from_pydict({"groups": groups, "input": input})
     daft_table = daft_table.eval_expression_list([col("groups"), col("input").cast(dtype)])
-    result = daft_table.agg([col("input").alias("list")._agg_list()], group_by=[col("groups")]).sort([col("groups")])
+    result = daft_table.agg([col("input").alias("list").agg_list()], group_by=[col("groups")]).sort([col("groups")])
     assert result.get_column("list").datatype() == DataType.list(dtype)
 
     input_as_dtype = daft_table.get_column("input").to_pylist()
@@ -570,16 +674,15 @@ def test_grouped_pyobj_list_aggs() -> None:
     expected_idx = [[1, 3], [4, 5], [0, 2]]
 
     daft_table = MicroPartition.from_pydict({"groups": groups, "input": input})
-    result = daft_table.agg([col("input").alias("list")._agg_list()], group_by=[col("groups")]).sort([col("groups")])
+    result = daft_table.agg([col("input").alias("list").agg_list()], group_by=[col("groups")]).sort([col("groups")])
     expected_groups = [[input[i] for i in group] for group in expected_idx]
     assert result.to_pydict() == {"groups": [1, 2, None], "list": expected_groups}
 
 
 def test_list_aggs_empty() -> None:
-
     daft_table = MicroPartition.from_pydict({"col_A": [], "col_B": []})
     daft_table = daft_table.agg(
-        [col("col_A").cast(DataType.int32()).alias("list")._agg_list()],
+        [col("col_A").cast(DataType.int32()).alias("list").agg_list()],
         group_by=[col("col_B")],
     )
     assert daft_table.get_column("list").datatype() == DataType.list(DataType.int32())
@@ -605,7 +708,7 @@ def test_global_concat_aggs(dtype, with_null) -> None:
     daft_table = MicroPartition.from_pydict({"input": input}).eval_expression_list(
         [col("input").cast(DataType.list(dtype))]
     )
-    concated = daft_table.agg([col("input").alias("concat")._agg_concat()])
+    concated = daft_table.agg([col("input").alias("concat").agg_concat()])
     assert concated.get_column("concat").datatype() == DataType.list(dtype)
 
     input_as_dtype = daft_table.get_column("input").to_pylist()
@@ -624,7 +727,7 @@ def test_global_concat_aggs_pyobj() -> None:
     ]
 
     table = MicroPartition.from_pydict({"input": input})
-    concatted = table.agg([col("input").alias("concat")._agg_concat()])
+    concatted = table.agg([col("input").alias("concat").agg_concat()])
     assert concatted.get_column("concat").datatype() == DataType.python()
     assert concatted.to_pydict()["concat"] == [expected]
 
@@ -643,7 +746,7 @@ def test_grouped_concat_aggs(dtype) -> None:
     daft_table = MicroPartition.from_pydict({"groups": groups, "input": input}).eval_expression_list(
         [col("groups"), col("input").cast(DataType.list(dtype))]
     )
-    concat_grouped = daft_table.agg([col("input").alias("concat")._agg_concat()], group_by=[col("groups") % 2]).sort(
+    concat_grouped = daft_table.agg([col("input").alias("concat").agg_concat()], group_by=[col("groups") % 2]).sort(
         [col("groups")]
     )
     assert concat_grouped.get_column("concat").datatype() == DataType.list(dtype)
@@ -667,7 +770,7 @@ def test_grouped_concat_aggs_pyobj() -> None:
     ]
 
     table = MicroPartition.from_pydict({"input": input, "groups": [1, 2, 3, 3, 4]})
-    concatted = table.agg([col("input").alias("concat")._agg_concat()], group_by=[col("groups")]).sort([col("groups")])
+    concatted = table.agg([col("input").alias("concat").agg_concat()], group_by=[col("groups")]).sort([col("groups")])
     assert concatted.get_column("concat").datatype() == DataType.python()
     assert concatted.to_pydict() == {
         "groups": [1, 2, 3, 4],
@@ -681,10 +784,9 @@ def test_grouped_concat_aggs_pyobj() -> None:
 
 
 def test_concat_aggs_empty() -> None:
-
     daft_table = MicroPartition.from_pydict({"col_A": [], "col_B": []})
     daft_table = daft_table.agg(
-        [col("col_A").cast(DataType.list(DataType.int32())).alias("concat")._agg_concat()],
+        [col("col_A").cast(DataType.list(DataType.int32())).alias("concat").agg_concat()],
         group_by=[col("col_B")],
     )
 
